# -*- coding: utf-8 -*-

# gracie/__init__.py
# Part of Gracie, an OpenID provider.
#
# Copyright © 2007–2009 Ben Finney <ben+python@benfinney.id.au>
# This is free software; you may copy, modify and/or distribute this work
# under the terms of the GNU General Public License, version 2 or later.
# No warranty expressed or implied. See the file ‘LICENSE.GPL-2’ for details.

""" Gracie - OpenID provider for local accounts.

    Gracie is an OpenID server (a "provider" in OpenID terminology) that
    serves OpenID identities for the local system PAM accounts. It
    authenticates users with a username/password challenge.

    The OpenID protocol is documented at <URL:http://openid.net/>.

    """

import version

_version = version.version
_copyright = version.copyright
_license = version.license
_url = "http://trac.whitetree.org/gracie/"



# -*- coding: utf-8 -*-

# test/test_httpresponse.py
# Part of Gracie, an OpenID provider.
#
# Copyright © 2007–2009 Ben Finney <ben+python@benfinney.id.au>
# This is free software; you may copy, modify and/or distribute this work
# under the terms of the GNU General Public License, version 2 or later.
# No warranty expressed or implied. See the file ‘LICENSE.GPL-2’ for details.

""" Unit test for httpresponse module.
    """

import sys
from StringIO import StringIO

import scaffold
from scaffold import Mock
from test_server import (
    Stub_ResponseHeader,
    )

from gracie import httpresponse


class ResponseHeader_TestCase(scaffold.TestCase):
    """ Test cases for ResponseHeader class. """

    def setUp(self):
        """ Set up test fixtures """

        self.header_class = httpresponse.ResponseHeader

        self.valid_headers = {
            'simple': dict(
                code = 200,
                ),
            'ok': dict(
                code = 200,
                ),
            'ok-protocol': dict(
                code = 200,
                protocol = "HTTP/1.1",
                ),
            'content-type-bogus': dict(
                code = 200,
                content_type = "BoGuS",
                ),
            }

        for key, params in self.valid_headers.items():
            args = params.get('args', dict())
            code = params['code']
            args['code'] = code
            protocol = params.get('protocol')
            if protocol is not None:
                args['protocol'] = protocol
            content_type = params.get('content_type')
            if content_type is not None:
                args['content_type'] = content_type
            params['args'] = args
            instance = self.header_class(**args)
            params['instance'] = instance

    def test_instantiate(self):
        """ New ResponseHeader instance should be created """
        for params in self.valid_headers.values():
            instance = params['instance']
            self.failIfIs(None, instance)

    def test_code_as_specified(self):
        """ ResponseHeader should have specified status code """
        for params in self.valid_headers.values():
            code = params['code']
            instance = params['instance']
            self.failUnlessEqual(code, instance.code)

    def test_protocol_default_http_1_0(self):
        """ ResponseHeader protocol should default to HTTP/1.0 """
        params = self.valid_headers['simple']
        instance = params['instance']
        protocol = "HTTP/1.0"
        self.failUnlessEqual(protocol, instance.protocol)

    def test_protocol_as_specified(self):
        """ ResponseHeader should have specified response protocol """
        for params in self.valid_headers.values():
            protocol = params.get('protocol')
            if protocol is None:
                continue
            instance = params['instance']
            self.failUnlessEqual(protocol, instance.protocol)

    def test_content_type_default_xhtml(self):
        """ ResponseHeader should default to Content-Type of XHTML """
        params = self.valid_headers['simple']
        instance = params['instance']
        expect_field =("Content-Type", "application/xhtml+xml")
        self.failUnless(expect_field in instance.fields)

    def test_content_type_as_specified(self):
        """ ResponseHeader should have specified Content-Type field """
        for params in self.valid_headers.values():
            content_type = params.get('content_type')
            expect_field = ("Content-Type", content_type)
            if content_type is None:
                continue
            instance = params['instance']
            self.failUnless(expect_field in instance.fields)


class Stub_RequestHandler(object):
    """ Stub class for BaseHTTPRequestHandler """

    def __init__(self):
        self.wfile = StringIO("")
    def send_response(self, code, message=None):
        pass
    def end_headers(self):
        pass

class Response_TestCase(scaffold.TestCase):
    """ Test cases for Response class. """

    def setUp(self):
        """ Set up test fixtures """
        self.mock_tracker = scaffold.MockTracker()

        self.response_class = httpresponse.Response

        self.valid_responses = {
            'simple': dict(
                header = Stub_ResponseHeader(code = 200),
                ),
            'payload': dict(
                header = Stub_ResponseHeader(code = 200),
                data = object(),
                ),
            }

        for key, params in self.valid_responses.items():
            args = params.get('args', dict())
            header = params['header']
            args['header'] = header
            data = params.get('data')
            if data is not None:
                args['data'] = data
            params['args'] = args
            instance = self.response_class(**args)
            params['instance'] = instance

    def test_initialise(self):
        """ New Response instance should be created """
        for params in self.valid_responses.values():
            instance = params['instance']
            self.failIfIs(None, instance)

    def test_header_as_specified(self):
        """ Response should have specified header """
        for params in self.valid_responses.values():
            header = params['header']
            instance = params['instance']
            self.failUnlessEqual(header, instance.header)

    def test_data_as_specified(self):
        """ Response should have specified data """
        params = self.valid_responses['payload']
        data = params['data']
        instance = params['instance']
        self.failUnlessEqual(data, instance.data)

    def test_send_to_handler_uses_handler(self):
        """ Response.send_to_handler should use specified handler """
        for params in self.valid_responses.values():
            instance = params['instance']
            handler = Mock(
                'HTTPRequestHandler',
                tracker=self.mock_tracker)
            instance.send_to_handler(handler)
            expect_mock_output = """\
                Called HTTPRequestHandler.send_response(...)
                ...Called HTTPRequestHandler.end_headers()
                Called HTTPRequestHandler.wfile.write(...)
                Called HTTPRequestHandler.wfile.close()
                """
            self.failUnlessMockCheckerMatch(expect_mock_output)



#!/usr/bin/env python
"""This file is automatically generated by generate_version_info
It uses the current working tree to determine the revision.
So don't edit it. :)
"""

version_info = {'branch_nick': u'gracie.version-0.2.10',
 'build_date': '2009-10-05 18:35:26 +1100',
 'clean': None,
 'date': '2009-10-05 18:26:56 +1100',
 'revision_id': 'ben@benfinney.id.au-20091005072656-p8z1txbrpbiktq1l',
 'revno': 155}

revisions = {}

file_revisions = {}



if __name__ == '__main__':
    print 'revision: %(revno)d' % version_info
    print 'nick: %(branch_nick)s' % version_info
    print 'revision id: %(revision_id)s' % version_info



# -*- coding: utf-8 -*-

# gracie/httpserver.py
# Part of Gracie, an OpenID provider.
#
# Copyright © 2007–2009 Ben Finney <ben+python@benfinney.id.au>
# This is free software; you may copy, modify and/or distribute this work
# under the terms of the GNU General Public License, version 2 or later.
# No warranty expressed or implied. See the file ‘LICENSE.GPL-2’ for details.

""" Behaviour for HTTP server portion of Gracie.
    """

import logging
import urlparse
from BaseHTTPServer import HTTPServer

# Get the Python logging instance for this module
_logger = logging.getLogger("gracie.httpserver")

default_host = "localhost"
default_port = 8000


class BaseHTTPServer(HTTPServer, object):
    """ Shim to insert base object type into hierarchy """


http_port = 80

def net_location(host, port=None):
    """ Construct a location string from host string and port number """
    if port is None or port == http_port:
        location_spec = "%(host)s"
    else:
        location_spec = "%(host)s:%(port)s"
    location = location_spec % vars()
    return location

default_root_url = urlparse.urlunsplit(
    ("http", net_location(default_host, default_port), "/", "", "")
    )


class HTTPServer(BaseHTTPServer):
    """ Server for HTTP protocol requests """

    def __init__(
        self,
        server_address, RequestHandlerClass, gracie_server
        ):
        """ Set up a new instance """
        self.gracie_server = gracie_server
        self._setup_version()
        self._setup_logging()
        super(HTTPServer, self).__init__(
            server_address, RequestHandlerClass
            )
        self._log_location()

    def _setup_version(self):
        """ Set up version string for this server """
        self.version = self.gracie_server.version

    def _setup_logging(self):
        """ Set up logging for this server """
        version = self.version
        _logger.info(
            "Starting Gracie HTTP server (version %(version)s)"
            % vars()
            )

    def server_bind(self):
        """ Bind and name the server """
        super(HTTPServer, self).server_bind()
        self.server_location = net_location(
            self.server_name, self.server_port
            )

    def _log_location(self):
        """ Log the net location of the server """
        location = net_location(*self.server_address)
        _logger.info(
            "Listening on address %(location)s"
            % vars()
            )

    def handle_request(self):
        """ Handle a single request """
        try:
            super(HTTPServer, self).handle_request()
        except (KeyboardInterrupt, SystemExit), e:
            exc_name = e.__class__.__name__
            message = "Received %(exc_name)s" % vars()
            _logger.warn(message)
            raise
        except Exception, e:
            message = str(e)
            _logger.error(message)
            raise



# -*- coding: utf-8 -*-

# test/test_server.py
# Part of Gracie, an OpenID provider.
#
# Copyright © 2007–2009 Ben Finney <ben+python@benfinney.id.au>
# This is free software; you may copy, modify and/or distribute this work
# under the terms of the GNU General Public License, version 2 or later.
# No warranty expressed or implied. See the file ‘LICENSE.GPL-2’ for details.

""" Unit test for server module.
    """

import optparse

import scaffold

from gracie import server


def stub_server_bind(server):
    """ Stub method to get server location """
    (host, port) = server.server_address
    (server.server_name, server.server_port) = (host, port)

class Stub_HTTPServer(object):
    """ Stub class for HTTPServer """
    def __init__(
        self,
        server_address, RequestHandlerClass, gracie_server
        ):
        """ Set up a new instance """

    server_bind = stub_server_bind

class Stub_ResponseHeader(object):
    """ Stub class for response header """

    def __init__(self, code, protocol=None):
        self.code = code
        self.protocol = protocol
        self.fields = dict()

class Stub_Response(object):
    """ Stub class for Response """

    def __init__(self, header, data=None):
        """ Set up a new instance """
        self.header = header
        self.data = data


class Stub_ConsumerAuthStore(object):
    """ Stub class for ConsumerAuthStore """

    def __init__(self):
        self._authorisations = dict()

    def store_authorisation(self, auth_tuple, status):
        self._authorisations[auth_tuple] = status

    def is_authorised(self, auth_tuple):
        return self._authorisations.get(auth_tuple, False)

class Stub_ConsumerAuthStore_always_auth(Stub_ConsumerAuthStore):
    """ ConsumerAuthStore stub that always authorises """

    def is_authorised(self, auth_tuple):
        return True

class Stub_ConsumerAuthStore_never_auth(Stub_ConsumerAuthStore):
    """ ConsumerAuthStore stub that never authorises """

    def is_authorised(self, auth_tuple):
        return False


class Stub_SessionManager(object):
    """ Stub class for SessionManager """

    def store_session(self, session):
        pass

    def get_session(self, session_id):
        return None

    def remove_session(self, session_id):
        pass


class Stub_HTTPRequestHandler(object):
    """ Stub class for HTTPRequestHandler """


class Stub_OpenIDError(Exception):
    """ Stub error class for openid module """

class Stub_OpenIDStore(object):
    """ Stub class for openid backing store """

    def __init__(self, _, *args, **kwargs):
        """ Set up a new instance """

class Stub_OpenIDServer(object):
    """ Stub class for an OpenID protocol server """

    def __init__(self, store):
        """ Set up a new instance """

    def decodeRequest(self, request):
        return Stub_OpenIDResponse()

    def encodeResponse(self, response):
        return Stub_OpenIDWebResponse()

class Stub_OpenIDRequest(object):
    """ Stub class for an OpenID protocol request """

    def __init__(self, http_query, params=None):
        """ Set up a new instance """

        self.mode = http_query.get('openid.mode')
        keys = [
            'identity', 'trust_root', 'immediate', 'return_to',
            ]
        if params is None:
            params = dict()
        for key in keys:
            setattr(self, key, None)
            if key in params:
                setattr(self, key, params[key])

    def answer(self, allow, server_url=None):
        response = Stub_OpenIDResponse(dict(
            allow = allow,
            server_url = server_url,
            ))
        return response

class Stub_OpenIDResponse(object):
    """ Stub class for an OpenID protocol response """

    def __init__(self, params=None):
        self.params = params

    def encodeToURL(self):
        url = "http://stub/openid_response/" + ";".join(
            "%s=%s" % (key, val) for key, val in self.params.items()
            )
        return url

class Stub_OpenIDWebResponse(object):
    """ Stub class for an encoded OpenID response """

    def __init__(self):
        """ Set up a new instance """
        self.code = 200
        self.headers = {"openid": "yes"}
        self.body = "OpenID response"

def make_default_opts():
    """ Create commandline opts instance with required values """
    opts = optparse.Values(dict(
        datadir = "/tmp",
        host = "example.org", port = 9779,
        ))
    return opts


class GracieServer_TestCase(scaffold.TestCase):
    """ Test cases for GracieServer class. """

    def setUp(self):
        """ Set up test fixtures """
        self.mock_tracker = scaffold.MockTracker()

        self.server_class = server.GracieServer

        scaffold.mock(
            "server.OpenIDServer",
            mock_obj=Stub_OpenIDServer,
            tracker=self.mock_tracker)
        scaffold.mock(
            "server.OpenIDStore",
            mock_obj=Stub_OpenIDStore,
            tracker=self.mock_tracker)
        scaffold.mock(
            "server.ConsumerAuthStore",
            mock_obj=Stub_ConsumerAuthStore,
            tracker=self.mock_tracker)
        scaffold.mock(
            "server.SessionManager",
            mock_obj=Stub_SessionManager,
            tracker=self.mock_tracker)

        scaffold.mock(
            "server.HTTPServer",
            mock_obj=Stub_HTTPServer,
            tracker=self.mock_tracker)
        scaffold.mock(
            "server.HTTPRequestHandler",
            mock_obj=Stub_HTTPRequestHandler,
            tracker=self.mock_tracker)

        self.valid_servers = {
            'simple': dict(
                ),
            'with-opts': dict(
                opts = dict(
                    foo="spam",
                    bar="eggs",
                    ),
                ),
            'datadir': dict(
                opts = dict(
                    datadir = "/foo/bar",
                    ),
                datadir = "/foo/bar",
                ),
            }

        for key, params in self.valid_servers.items():
            args = params.get('args')
            if not args:
                args = dict()
            opts = make_default_opts()
            opts._update_loose(params.get('opts', dict()))
            params['opts'] = opts
            args.update(dict(
                socket_params=None,
                opts=opts,
                ))
            instance = self.server_class(**args)
            params['args'] = args
            params['instance'] = instance

    def tearDown(self):
        """ Tear down test fixtures """
        scaffold.mock_restore()

    def test_instantiate(self):
        """ New GracieServer instance should be created """
        for params in self.valid_servers.values():
            instance = params['instance']
            self.failIfIs(None, instance)

    def test_version_as_specified(self):
        """ GracieServer should have specified version string """
        params = self.valid_servers['simple']
        scaffold.mock(
            "server.version", tracker=self.mock_tracker)
        version_test = "1.414.test"
        server.version.version_full = version_test
        instance = self.server_class(**params['args'])
        self.failUnlessEqual(version_test, instance.version)

    def test_opts_as_specified(self):
        """ GracieServer should have specified opts mapping """
        params = self.valid_servers['with-opts']
        instance = params['instance']
        opts = params['opts']
        self.failUnlessEqual(opts, instance.opts)

    def test_server_creates_http_server(self):
        """ GracieServer should create an HTTP server """
        params = self.valid_servers['simple']
        args = params['args']
        opts = params['opts']
        server_address = (opts.host, opts.port)
        scaffold.mock(
            "server.HTTPServer", tracker=self.mock_tracker)
        expect_mock_output = """\
            Called server.HTTPServer(
                %(server_address)r,
                <class '...HTTPRequestHandler'>,
                <gracie.server.GracieServer object ...>)
            """ % vars()
        instance = self.server_class(**params['args'])
        self.failUnlessMockCheckerMatch(expect_mock_output)

    def test_server_has_openid_server(self):
        """ GracieServer should have an openid_server attribute """
        params = self.valid_servers['simple']
        instance = params['instance']
        openid_server = instance.openid_server
        self.failUnless(isinstance(openid_server, Stub_OpenIDServer))

    def test_openid_store_created_with_datadir(self):
        """ OpenIDStore should be created with specified datadir """
        params = self.valid_servers['datadir']
        datadir = params['datadir']
        scaffold.mock(
            "server.OpenIDStore", tracker=self.mock_tracker)
        expect_mock_output = """\
            Called server.OpenIDStore(%(datadir)r)
            """ % vars()
        instance = self.server_class(**params['args'])
        self.failUnlessMockCheckerMatch(expect_mock_output)

    def test_server_has_auth_service(self):
        """ GracieServer should have an auth_service attribute """
        params = self.valid_servers['simple']
        instance = params['instance']
        auth_service = instance.auth_service
        self.failIfIs(None, auth_service)

    def test_server_has_session_manager(self):
        """ GracieServer should have a sess_manager attribute """
        params = self.valid_servers['simple']
        instance = params['instance']
        sess_manager = instance.sess_manager
        self.failIfIs(None, sess_manager)

    def test_server_has_authorisation_store(self):
        """ GracieServer should have a consumer_auth_store attribute """
        params = self.valid_servers['simple']
        instance = params['instance']
        consumer_auth_store = instance.consumer_auth_store
        self.failIfIs(None, consumer_auth_store)

    def test_serve_forever_is_callable(self):
        """ GracieServer.serve_forever should be callable """
        self.failUnless(callable(self.server_class.serve_forever))



# -*- coding: utf-8 -*-

# test/test_httpserver.py
# Part of Gracie, an OpenID provider.
#
# Copyright © 2007–2009 Ben Finney <ben+python@benfinney.id.au>
# This is free software; you may copy, modify and/or distribute this work
# under the terms of the GNU General Public License, version 2 or later.
# No warranty expressed or implied. See the file ‘LICENSE.GPL-2’ for details.

""" Unit test for httpserver module.
    """

import sys
from StringIO import StringIO

import scaffold
from scaffold import Mock
from test_server import stub_server_bind, make_default_opts
from test_gracied import Stub_GracieServer

from gracie import httpserver


class net_location_TestCase(scaffold.TestCase):
    """ Test cases for net_location function. """

    def setUp(self):
        """ Set up test fixtures """
        self.default_port_prev = httpserver.default_port
        self.default_port_test = 9779
        httpserver.default_port = self.default_port_test

    def tearDown(self):
        """ Tear down test fixtures """
        httpserver.default_port = self.default_port_prev

    def test_combines_components_to_location(self):
        """ net_location() should combine components to net location """
        http_port = 80
        locations = {
            ("foo", None): "foo",
            ("foo", http_port): "foo",
            ("foo", self.default_port_test):
                "foo:%s" % self.default_port_test,
            ("foo", 2468): "foo:2468",
            }

        for (host, port), expect_location in locations.items():
            location = httpserver.net_location(host, port)
            self.failUnlessEqual(expect_location, location)


class Stub_HTTPRequestHandler(object):
    """ Stub class for HTTPRequestHandler """


class HTTPServer_TestCase(scaffold.TestCase):
    """ Test cases for HTTPServer class. """

    def setUp(self):
        """ Set up test fixtures """
        self.mock_tracker = scaffold.MockTracker()

        self.server_class = httpserver.HTTPServer

        self.stub_handler_class = Stub_HTTPRequestHandler
        self.mock_handler_class = Mock('HTTPRequestHandler')

        self.server_bind_prev = httpserver.BaseHTTPServer.server_bind
        httpserver.BaseHTTPServer.server_bind = stub_server_bind

        self.stdout_prev = sys.stdout
        self.stdout_test = StringIO()
        sys.stdout = self.stdout_test

        self.valid_servers = {
            'simple': dict(
                address = ('', 80),
                ),
            }

        for key, params in self.valid_servers.items():
            args = params.get('args')
            address = params.get('address')
            handler_class = params.setdefault(
                'handler_class', self.stub_handler_class)
            opts = make_default_opts()
            gracie_server = params.setdefault(
                'gracie_server', Stub_GracieServer(address, opts)
                )
            if not args:
                args = dict(
                    server_address = params['address'],
                    RequestHandlerClass = params['handler_class'],
                    gracie_server = params['gracie_server'],
                    )
            instance = self.server_class(**args)
            params['args'] = args
            params['instance'] = instance

    def tearDown(self):
        """ Tear down test fixtures """
        sys.stdout = self.stdout_prev
        httpserver.BaseHTTPServer.server_bind = self.server_bind_prev
        scaffold.mock_restore()

    def test_instantiate(self):
        """ New HTTPServer instance should be created """
        for params in self.valid_servers.values():
            instance = params['instance']
            self.failIfIs(None, instance)

    def test_version_as_specified(self):
        """ HTTPServer should have specified version string """
        params = self.valid_servers['simple']
        server_module = sys.modules['gracie.server']
        scaffold.mock(
            "server_module.version", tracker=self.mock_tracker)
        version_test = Stub_GracieServer.version
        server_module.version.version_full = version_test
        instance = self.server_class(**params['args'])
        self.failUnlessEqual(version_test, instance.version)

    def test_request_handler_class_as_specified(self):
        """ HTTPServer should have specified RequestHandlerClass """
        for params in self.valid_servers.values():
            instance = params['instance']
            handler_class = params['handler_class']
            self.failUnlessEqual(
                handler_class, instance.RequestHandlerClass
                )

    def test_gracie_server_as_specified(self):
        """ HTTPServer should have specified Gracie server instance """
        for params in self.valid_servers.values():
            instance = params['instance']
            gracie_server = params['gracie_server']
            self.failUnlessEqual(
                gracie_server, instance.gracie_server
                )

    def test_serve_forever_is_callable(self):
        """ HTTPServer.serve_forever should be callable """
        self.failUnless(callable(self.server_class.serve_forever))



# -*- coding: utf-8 -*-

# gracie/server.py
# Part of Gracie, an OpenID provider.
#
# Copyright © 2007–2009 Ben Finney <ben+python@benfinney.id.au>
# This is free software; you may copy, modify and/or distribute this work
# under the terms of the GNU General Public License, version 2 or later.
# No warranty expressed or implied. See the file ‘LICENSE.GPL-2’ for details.

""" Behaviour for OpenID provider server.
    """

import logging
from openid.server.server import Server as OpenIDServer
from openid.store.filestore import FileOpenIDStore as OpenIDStore

from httprequest import HTTPRequestHandler
from httpserver import HTTPServer
from authservice import PamAuthService as AuthService
from authorisation import ConsumerAuthStore
from session import SessionManager
import version

# Get the Python logging instance for this module
_logger = logging.getLogger("gracie.server")


class GracieServer(object):
    """ Server for Gracie OpenID provider service """

    def __init__(self, socket_params, opts):
        """ Set up a new instance """
        self.version = version.version_full
        self.opts = opts
        self._setup_logging()
        server_address = (opts.host, opts.port)
        self.httpserver = HTTPServer(
            server_address, HTTPRequestHandler, self
            )
        self._setup_openid()
        self.auth_service = AuthService()
        self.sess_manager = SessionManager()
        self.consumer_auth_store = ConsumerAuthStore()

    def _setup_openid(self):
        """ Set up OpenID parameters """
        store = OpenIDStore(self.opts.datadir)
        self.openid_server = OpenIDServer(store)

    def __del__(self):
        _logger.info("Exiting Gracie server")

    def _setup_logging(self):
        """ Set up logging for this server """
        server_version = self.version
        _logger.info(
            "Starting Gracie server (version %(server_version)s)"
            % vars()
            )

    def serve_forever(self):
        """ Begin serving requests indefinitely """
        self.httpserver.serve_forever()



# -*- coding: utf-8 -*-

# gracie/httprequest.py
# Part of Gracie, an OpenID provider.
#
# Copyright © 2007–2009 Ben Finney <ben+python@benfinney.id.au>
# This is free software; you may copy, modify and/or distribute this work
# under the terms of the GNU General Public License, version 2 or later.
# No warranty expressed or implied. See the file ‘LICENSE.GPL-2’ for details.

""" HTTP request handling.
    """

import BaseHTTPServer
import logging
import time
import cgi
import Cookie
import urlparse
import routes
from openid.server.server import BROWSER_REQUEST_MODES

from gracie import pagetemplate
from gracie.httpresponse import ResponseHeader, Response
from gracie.httpresponse import response_codes as http_codes
from gracie.authservice import AuthenticationError

session_cookie_name = "gracie_session"

_logger = logging.getLogger("gracie.httprequest")


class BaseHTTPRequestHandler(
    BaseHTTPServer.BaseHTTPRequestHandler,
    object):
    """ Shim to insert base object type into hierarchy """


mapper = routes.Mapper()
mapper.connect('root', '', controller='about', action='view')
mapper.connect('openid', 'openidserver', controller='openid')
mapper.connect('identity', 'id/:name',
               controller='identity', action='view')
mapper.connect('login', 'login', controller='login', action='view')
mapper.connect('logout', 'logout', controller='logout', action='view')

class HTTPRequestHandler(BaseHTTPRequestHandler):
    """ Handler for individual HTTP requests """

    def __init__(self, request, client_address, server):
        """ Set up a new instance """
        self.gracie_server = server.gracie_server
        self._setup_version()
        super(HTTPRequestHandler, self).__init__(
            request, client_address, server
            )

    def _setup_version(self):
        """ Set up the version string """
        version = self.gracie_server.version
        self.server_version = "Gracie/%(version)s" % vars()

    def log_message(self, format, *args, **kwargs):
        """ Log a message via the server's logger """
        logger = _logger
        loglevel = logging.INFO
        logger.log(loglevel, format, *args, **kwargs)

    def _make_server_url(self, path):
        """ Construct a URL to a path on this server """
        root_url = self.gracie_server.opts.root_url
        url = urlparse.urljoin(root_url, path)
        return url

    def _begin_new_session(self):
        """ Begin a new server session """
        sess_manager = self.gracie_server.sess_manager
        self.session = dict()
        session_id = sess_manager.create_session(self.session)
        self.session['session_id'] = session_id

    def _authenticate_session(self, username):
        """ Authenticate the current session as specified username """
        if username is not None:
            auth_entry = self.gracie_server.auth_service.get_entry(username)
            self.session.update(dict(
                username = username,
                auth_entry = auth_entry,
                ))
            _logger.info(
                "Session authenticated as %(username)r" % vars()
                )
        else:
            _logger.info("Session not authenticated")

    def _setup_auth_session(self):
        """ Set up the authentication session """
        sess_manager = self.gracie_server.sess_manager
        session_id = self._get_auth_cookie()
        try:
            self.session = sess_manager.get_session(session_id)
        except KeyError:
            self._begin_new_session()

        username = self.session.get('username')
        self._authenticate_session(username)

    def _remove_auth_session(self):
        """ Remove the authentication session """
        sess_manager = self.gracie_server.sess_manager
        if self.session:
            sess_manager.remove_session(self.session.get('session_id'))
        self._begin_new_session()

        _logger.info("Removed authentication session")

    def _get_cookie(self, name):
        """ Get a cookie from the request """
        value = None
        cookie_string = ";".join(self.headers.getheaders('Cookie'))
        cookies = Cookie.BaseCookie(cookie_string)
        if cookies:
            if session_cookie_name in cookies:
                value = cookies[session_cookie_name].value
        return value

    def _get_auth_cookie(self):
        """ Get the authentication cookie from the request """
        session_id = self._get_cookie('session')
        return session_id

    def _set_cookie(self, response, name, value, expire=None):
        """ Set a cookie in the response header """
        field_name = "Set-Cookie"
        field_value = "%(name)s=%(value)s" % vars()
        if expire:
            field_value = (
                "%(field_value)s;Expires=%(expire)s" % vars())
        field = (field_name, field_value)
        response.header.fields.append(field)

    def _set_auth_cookie(self, response):
        """ Set the authentication cookie in the response """
        field_name = "Set-Cookie"
        epoch = time.gmtime(0)
        expire_immediately = time.strftime(
            '%a, %d-%b-%y %H:%M:%S GMT', epoch)

        session_id = self.session['session_id']
        self._set_cookie(response, session_cookie_name, session_id)

    def _get_username_from_identity(self, identity):
        """ Parse a local username from an OpenID URL """
        name = None
        (_, _, path, _, _) = urlparse.urlsplit(identity)
        route_map = mapper.match(path)
        if route_map:
            if route_map.get('controller') == 'identity':
                name = route_map.get('name')
        return name

    def _make_openid_url(self, username):
        """ Generate the OpenID URL for a username """
        path = "id/%(username)s" % vars()
        url = self._make_server_url(path)
        return url

    def _get_session_openid_url(self):
        """ Get the OpenID URL for the current session """
        openid_url = None
        if self.session:
            username = self.session.get('username')
            openid_url = self._make_openid_url(username)
            self.session['openid_url'] = openid_url
        return openid_url

    def _get_session_auth_entry(self):
        auth_entry = None
        if self.session:
            auth_entry = self.session.get('auth_entry')
        return auth_entry

    def _dispatch(self):
        """ Dispatch to the appropriate controller """
        controller_map = {
            None: self._make_url_not_found_error_response,
            'openid': self._handle_openid_request,
            'about': self._make_about_site_view_response,
            'identity': self._make_identity_view_response,
            'logout': self._make_logout_response,
            'login': self._make_login_response,
            }
        controller_name = None
        if self.route_map:
            controller_name = self.route_map['controller']
        controller = controller_map[controller_name]

        _logger.info(
            "Dispatching to controller %(controller_name)r" % vars()
            )

        response = controller()
        self._set_auth_cookie(response)
        self._send_response(response)

    def _send_response(self, response):
        """ Send an HTTP response to the user agent """
        response.send_to_handler(self)

        _logger.info("Sent HTTP response")

    def _parse_path(self):
        """ Parse the request path """
        keys = ('scheme', 'location', 'path', 'query', 'fragment')
        values = urlparse.urlsplit(self.path)
        self.parsed_url = dict(zip(keys, values))
        self.route_map = mapper.match(self.parsed_url['path'])

    def _parse_query(self):
        """ Parse query fields from the request data """
        self.query = {}
        for key, value in cgi.parse_qsl(self.query_data):
            self.query[key] = value

    def handle(self):
        """ Handle the requests """
        try:
            super(HTTPRequestHandler, self).handle()
        except (KeyboardInterrupt, SystemExit):
            raise
        except Exception, e:
            message = str(e)
            _logger.error(message)
            response = self._make_internal_error_response(message)
            self._send_response(response)
            raise

    def do_GET(self):
        """ Handle a GET request """
        self._setup_auth_session()
        self._parse_path()
        self.query_data = self.parsed_url['query']
        self._parse_query()
        self._dispatch()

    def do_POST(self):
        """ Handle a POST request """
        self._setup_auth_session()
        self.route_map = mapper.match(self.path)
        content_length = int(self.headers['Content-Length'])
        self.query_data = self.rfile.read(content_length)
        self._parse_query()
        self._dispatch()

    def _handle_openid_request(self):
        """ Handle a request to the OpenID server URL """
        openid_server = self.gracie_server.openid_server
        openid_request = openid_server.decodeRequest(self.query)
        if openid_request is None:
            response = self._make_about_site_view_response()
        else:
            _logger.info("Received OpenID protocol request")
            self.session['last_openid_request'] = openid_request
            if openid_request.mode in BROWSER_REQUEST_MODES:
                response = self._handle_openid_browser_request(
                    openid_request
                    )
            else:
                openid_response = openid_server.handleRequest(
                    openid_request
                    )
                response = self._make_response_from_openid_response(
                    openid_response
                    )

        return response

    def _handle_openid_browser_request(self, openid_request):
        """ Handle an OpenID request with user-agent interaction """
        is_session_identity = False
        identity = openid_request.identity
        if self.session:
            openid_url = self._get_session_openid_url()
            if openid_url:
                if openid_url == identity:
                    # Session is authenticated for requested identity
                    is_session_identity = True

        dispatch_map = {
            'checkid_immediate': self._make_checkid_immediate_response,
            'checkid_setup': self._make_checkid_setup_response,
            }
        response = dispatch_map[openid_request.mode](
            openid_request,
            is_session_identity=is_session_identity,
            )

        return response

    def _make_checkid_immediate_response(
        self, request,
        is_session_identity,
        ):
        """ Make a response for an OpenID checkid_immediate request """
        response = None

        if is_session_identity:
            openid_response = request.answer(True)
        else:
            openid_response = request.answer(
                False,
                self._make_server_url("openidserver")
                )
        response = self._make_response_from_openid_response(
            openid_response
            )

        return response

    def _make_checkid_setup_response(
        self, request,
        is_session_identity,
        ):
        """ Make a response for an OpenID checkid_setup request """
        response = None

        if is_session_identity:
            openid_response = request.answer(True)
            response = self._make_response_from_openid_response(
                openid_response
                )
        else:
            response = self._make_wrong_authentication_response(
                request.identity
                )

        return response

    def _make_response_from_openid_response(self, openid_response):
        """ Construct a response to a request to the OpenID server """
        _logger.info("Delegating response to OpenID library")
        openid_server = self.gracie_server.openid_server
        web_response = openid_server.encodeResponse(openid_response)
        header = ResponseHeader(web_response.code)
        for name, value in web_response.headers.items():
            field = (name, value)
            header.fields.append(field)
        response = Response(header, web_response.body)

        return response

    def _make_url_from_openid_response(self, openid_response):
        """ Construct a URL corresponding to an OpenID response """
        _logger.info("Getting URL for OpenID response")
        url = openid_response.encodeToURL()
        return url

    def _make_redirect_response(self, url):
        """ Construct a response for a redirect """
        header = ResponseHeader(http_codes["Found"])
        header.fields.append(("Location", url))
        data = ""
        response = Response(header, data)
        _logger.info("Redirecting to %(url)r" % vars())
        return response

    def _get_page_data(self, page):
        """ Get the actual data to be used from a page """
        page.values.update(dict(
            server_version = self.server_version,
            server_location = self.server.server_location,
            auth_entry = self._get_session_auth_entry(),
            openid_url = self._get_session_openid_url(),
            root_url = self._make_server_url(""),
            server_url = self._make_server_url("openidserver"),
            login_url = self._make_server_url("login"),
            logout_url = self._make_server_url("logout"),
            ))
        return page.serialise()

    def _make_internal_error_response(self, message):
        """ Construct an Internal Error error response """
        header = ResponseHeader(http_codes["Internal Server Error"])
        page = pagetemplate.internal_error_page(message)
        data = self._get_page_data(page)
        response = Response(header, data)
        return response

    def _make_url_not_found_error_response(self):
        """ Construct a Not Found error response """
        header = ResponseHeader(http_codes["Not Found"])
        page = pagetemplate.url_not_found_page(self.path)
        data = self._get_page_data(page)
        response = Response(header, data)
        return response

    def _make_protocol_error_response(self, message):
        """ Construct a Protocol Error error response """
        header = ResponseHeader(http_codes["OK"])
        page = pagetemplate.protocol_error_page(message)
        data = self._get_page_data(page)
        response = Response(header, data)
        return response

    def _make_about_site_view_response(self):
        """ Construct a response for the about-this-site view """
        header = ResponseHeader(http_codes["OK"])
        page = pagetemplate.about_site_view_page()
        data = self._get_page_data(page)
        response = Response(header, data)
        return response

    def _make_identity_view_response(self):
        """ Construct a response for an identity view """
        name = self.route_map['name']
        try:
            entry = self.gracie_server.auth_service.get_entry(name)
        except KeyError, e:
            entry = None

        if entry is None:
            header = ResponseHeader(http_codes["Not Found"])
            page = pagetemplate.identity_user_not_found_page(name)
        else:
            header = ResponseHeader(http_codes["OK"])
            identity_url = self._make_openid_url(name)
            page = pagetemplate.identity_view_user_page(
                entry, identity_url
                )

        data = self._get_page_data(page)
        response = Response(header, data)
        return response

    def _make_logout_response(self):
        """ Construct a response for a logout request """
        self._remove_auth_session()

        root_url = self._make_server_url("")
        response = self._make_redirect_response(root_url)
        return response

    def _make_login_response(self):
        """ Construct a response for a login request """
        controller = {
            'GET': self._make_login_view_response,
            'POST': self._make_login_submit_response,
            }[self.command]
        response = controller()
        return response

    def _make_login_view_response(self):
        """ Construct a response for a login view request """
        header = ResponseHeader(http_codes["OK"])
        page = pagetemplate.login_view_page()
        data = self._get_page_data(page)
        response = Response(header, data)
        return response

    def _make_login_submit_response(self):
        """ Construct a response for a login submit request """
        if 'cancel' in self.query:
            response = self._make_login_cancelled_response()
        elif 'submit' in self.query:
            response = self._make_login_authenticate_response()
        else:
            response = self._make_url_not_found_error_response()
        return response

    def _make_login_cancelled_response(self):
        """ Construct a response for a login cancel request """
        root_url = self._make_server_url("")
        openid_request = self.session.get('last_openid_request')
        if openid_request:
            return_url = self._make_url_from_openid_response(
                openid_request.answer(False)
                )
        else:
            return_url = root_url
        response = self._make_redirect_response(return_url)
        return response

    def _make_login_authenticate_response(self):
        """ Construct a response for a login authenticate request """
        want_username = self.query.get('username')
        password = self.query.get('password')
        credentials = dict(
            username=want_username,
            password=password
            )
        auth_service = self.gracie_server.auth_service
        try:
            username = auth_service.authenticate(credentials)
            authenticated = True
        except AuthenticationError, e:
            authenticated = False
        if authenticated:
            self._authenticate_session(username)
            response = self._make_login_succeeded_response()
        else:
            response = self._make_login_failed_response()
        return response

    def _make_login_failed_response(self):
        """ Construct a response for a failed login request """
        name = self.query.get('username')
        message = "The login details were incorrect."
        header = ResponseHeader(http_codes["OK"])
        page = pagetemplate.login_submit_failed_page(message, name)
        data = self._get_page_data(page)
        response = Response(header, data)
        return response

    def _make_login_succeeded_response(self):
        """ Construct a response for a successful login request """
        root_url = self._make_server_url("")
        openid_request = self.session.get('last_openid_request')
        openid_url = self._get_session_openid_url()
        if openid_request:
            if openid_request.identity == openid_url:
                openid_server = self.server.gracie_server.openid_server
                openid_response = openid_server.signatory.sign(
                    openid_request.answer(True)
                    )
                return_url = self._make_url_from_openid_response(
                    openid_response
                    )
                response = self._make_redirect_response(return_url)
            else:
                response = self._make_wrong_authentication_response(
                    openid_request.identity
                    )
        else:
            response = self._make_redirect_response(root_url)
        return response

    def _make_wrong_authentication_response(self, want_id):
        """ Make a response for action with wrong session auth """
        want_username = self._get_username_from_identity(want_id)
        header = ResponseHeader(http_codes["OK"])
        page = pagetemplate.wrong_authentication_page(
            want_username = want_username,
            want_id_url = want_id
            )
        data = self._get_page_data(page)
        response = Response(header, data)

        return response



# -*- coding: utf-8 -*-
#
# test/__init__.py
# Part of Gracie, an OpenID provider.
#
# Copyright © 2008–2009 Ben Finney <ben+python@benfinney.id.au>
# This is free software; you may copy, modify and/or distribute this work
# under the terms of the GNU General Public License, version 2 or later.
# No warranty expressed or implied. See the file ‘LICENSE.GPL-2’ for details.

""" Unit test suite for Gracie.
    """

import scaffold


suite = scaffold.make_suite()



# -*- coding: utf-8 -*-

# test/test_pagetemplate.py
# Part of Gracie, an OpenID provider.
#
# Copyright © 2007–2009 Ben Finney <ben+python@benfinney.id.au>
# This is free software; you may copy, modify and/or distribute this work
# under the terms of the GNU General Public License, version 2 or later.
# No warranty expressed or implied. See the file ‘LICENSE.GPL-2’ for details.

""" Unit test for pagetemplate module.
    """

from string import Template
import textwrap

import scaffold

from gracie import pagetemplate


def setup_PageTemplate_fixtures(testcase):
    """ Set up fixtures for PageTemplate test usage. """
    mock_page_template = Template(
        textwrap.dedent("""\
            Page {
                Coding: $character_encoding
                Title: $page_title
                $page_body
            }""")
        )

    mock_body_template = Template(
        textwrap.dedent("""\
            Body {
                Title: $page_title
                Auth entry: $auth_entry
                Content: $page_content
            }""")
        )

    scaffold.mock(
        "pagetemplate.page_template",
        mock_obj=mock_page_template)
    scaffold.mock(
        "pagetemplate.body_template",
        mock_obj=mock_body_template)


class Page_TestCase(scaffold.TestCase):
    """ Test cases for Page class. """

    def setUp(self):
        """ Set up test fixtures. """
        self.page_class = pagetemplate.Page
        setup_PageTemplate_fixtures(self)

        self.valid_pages = {
            'simple': dict(
                ),
            'welcome': dict(
                title = "Welcome",
                content = "Lorem ipsum dolor sic amet",
                ),
            'result': dict(
                title = "Result",
                content = "Here is your result: $result.",
                values = dict(
                    result = "thribble",
                    ),
                ),
            'authenticated': dict(
                title = "Logged in",
                content = "Lorem ipsum dolor sic amet",
                auth_entry = dict(
                    id = 1010,
                    name = "fred",
                    fullname = "Fred Nurk",
                    ),
                ),
            }

        for key, params in self.valid_pages.items():
            args = params.get('args')
            if args is None:
                args = dict()
            title = params.get('title')
            if title is not None:
                args.update(dict(title=title))
            params['args'] = args
            instance = self.page_class(**args)
            content = params.get('content')
            if content is not None:
                instance.content = content
            values = params.get('values', dict())
            if 'auth_entry' not in values:
                values.update({'auth_entry': None})
            values.update(dict(
                server_version = "FooBar v0.0",
                server_location = "frobnitz:9779",
                root_url = "/",
                server_url = "/openidserver",
                login_url = "/login",
                logout_url = "/logout",
                ))
            instance.values.update(values)
            params['instance'] = instance

    def tearDown(self):
        """ Tear down test fixtures. """
        scaffold.mock_restore()

    def test_instantiate(self):
        """ New Page instance should be created """
        for params in self.valid_pages.values():
            instance = params['instance']
            self.failIfIs(None, instance)

    def test_title_as_specified(self):
        """ Page title should be as specified """
        for params in self.valid_pages.values():
            title = params.get('title')
            if not title:
                continue
            instance = params['instance']
            self.failUnlessEqual(title, instance.title)

    def test_values_as_specified(self):
        """ Page values should be as specified """
        for params in self.valid_pages.values():
            values = params.get('values')
            if not values:
                continue
            instance = params['instance']
            self.failUnlessEqual(values, instance.values)

    def test_serialise_uses_template(self):
        """ Page.serialise should use provided template """
        params = self.valid_pages['welcome']
        instance = params['instance']
        expect_data = """\
            Page {
                Coding: ...
                Title: ...
                Body {
                Title: ...
                Auth entry: ...
                Content: ...
            }
            }"""
        page_data = instance.serialise()
        self.failUnlessOutputCheckerMatch(expect_data, page_data)

    def test_serialise_substitutes_values(self):
        """ Page.serialise should substitute values into template """
        params = self.valid_pages['result']
        instance = params['instance']
        title = params['title']
        content = params['content']
        values = params['values']
        auth_entry = values['auth_entry']
        expect_content = Template(content).substitute(values)
        expect_data = """\
            Page {
                Coding: utf-8
                Title: %(title)s
                Body {
                Title: %(title)s
                Auth entry: %(auth_entry)s
                Content: %(expect_content)s
            }
            }""" % vars()
        page_data = instance.serialise()
        self.failUnlessOutputCheckerMatch(expect_data, page_data)


class PageTemplates_TestCase(scaffold.TestCase):
    """ Test cases for individual page templates. """

    def setUp(self):
        """ Set up test fixtures. """
        setup_PageTemplate_fixtures(self)

    def tearDown(self):
        """ Tear down test fixtures. """
        scaffold.mock_restore()

    def test_internal_error_page_contains_message(self):
        """ Internal Error page should contain specified message """
        message = "Bad stuff happened"
        page = pagetemplate.internal_error_page(message)
        page_data = page.serialise()
        self.failUnlessOutputCheckerMatch(
            "Page {...%(message)s...}" % vars(), page_data
            )

    def test_url_not_found_page_contains_url(self):
        """ Resulting page should contain the referent URL """
        url = "/flim/flam/flom"
        page = pagetemplate.url_not_found_page(url)
        expect_data = "...%(url)s..." % vars()
        page_data = page.serialise()
        self.failUnlessOutputCheckerMatch(expect_data, page_data)

    def test_protocol_error_page_contains_message(self):
        """ Protocol Error page should contain specified message """
        message = "Bad stuff happened"
        page = pagetemplate.protocol_error_page(message)
        page_data = page.serialise()
        self.failUnlessOutputCheckerMatch(
            "Page {...%(message)s...}" % vars(), page_data
            )

    def test_about_site_page_returns_page(self):
        """ About Site page should return page """
        page = pagetemplate.about_site_view_page()
        page_data = page.serialise()
        self.failUnlessOutputCheckerMatch(
            "Page {...}", page_data
            )

    def test_identity_user_not_found_page_contains_name(self):
        """ Resulting page should contain the referent user name """
        name = "fred"
        page = pagetemplate.identity_user_not_found_page(name)
        expect_data = "...%(name)s..." % vars()
        page_data = page.serialise()
        self.failUnlessOutputCheckerMatch(expect_data, page_data)

    def test_identity_view_user_page_contains_name(self):
        """ Resulting page should contain the referent user name """
        entry = dict(
            id = 9999,
            name = "fred",
            fullname = "Fred Nurk",
            )
        identity_url = "http://example.org/id/%(name)s" % entry
        server_url = "http://example.org/openidserver"
        page = pagetemplate.identity_view_user_page(
            entry, identity_url
            )
        page.values.update(dict(server_url=server_url))
        page_data = page.serialise()
        self.failUnlessOutputCheckerMatch(
            "...%(id)s..." % entry, page_data
            )
        self.failUnlessOutputCheckerMatch(
            "...%(name)s..." % entry, page_data
            )
        self.failUnlessOutputCheckerMatch(
            "...%(fullname)s..." % entry, page_data
            )
        self.failUnlessOutputCheckerMatch(
            "...%(identity_url)s..." % vars(), page_data
            )

    def test_login_user_not_found_page_contains_name(self):
        """ Resulting page should contain the referent user name """
        name = "bogus"
        page = pagetemplate.login_user_not_found_page(name)
        page_data = page.serialise()
        self.failUnlessOutputCheckerMatch(
            "...%(name)s..." % vars(), page_data
            )

    def test_login_view_page_returns_page(self):
        """ View login should return page """
        page = pagetemplate.login_view_page()
        page_data = page.serialise()
        self.failUnlessOutputCheckerMatch(
            "Page {...}", page_data
            )

    def test_login_cancel_returns_page(self):
        """ Cancel login should return page """
        page = pagetemplate.login_cancelled_page()
        page_data = page.serialise()
        self.failUnlessOutputCheckerMatch(
            "Page {...}", page_data
            )

    def test_login_submit_failed_page_contains_details(self):
        """ Resulting page should contain specified details """
        message = "Did gyre and gimble in the wabe"
        name = "fred"
        page = pagetemplate.login_submit_failed_page(message, name)
        page_data = page.serialise()
        self.failUnlessOutputCheckerMatch(
            "...%(message)s..." % vars(), page_data
            )
        self.failUnlessOutputCheckerMatch(
            "...%(name)s..." % vars(), page_data
            )

    def test_wrong_authentication_page_contains_url(self):
        """ Resulting page should contain the specified URLs """
        want_username = "bill"
        want_id = "http://foo.example.org/id/bill"
        page = pagetemplate.wrong_authentication_page(
            want_username = want_username,
            want_id_url = want_id,
            )
        page_data = page.serialise()
        self.failUnlessOutputCheckerMatch(
            "...%(want_id)s..." % vars(), page_data
            )



# -*- coding: utf-8 -*-

# test/test_gracied.py
# Part of Gracie, an OpenID provider.
#
# Copyright © 2007–2009 Ben Finney <ben+python@benfinney.id.au>
# This is free software; you may copy, modify and/or distribute this work
# under the terms of the GNU General Public License, version 2 or later.
# No warranty expressed or implied. See the file ‘LICENSE.GPL-2’ for details.

""" Unit test for gracied daemon module.
    """

import sys
import os
from StringIO import StringIO
import daemon
from daemon import pidlockfile

import scaffold
from scaffold import Mock

module_name = 'gracied'
module_file_under_test = os.path.join(scaffold.bin_dir, module_name)
gracied = scaffold.make_module_from_file(
    module_name, module_file_under_test
    )


class Stub_GracieServer(object):
    """ Stub class for GracieServer """

    version = "3.14.test"

    def __init__(self, socket_params, opts):
        """ Set up a new instance """

    def serve_forever(self):
        pass


def setup_gracie_fixtures(testcase):
    """ Set up fixtures for Gracie test cases. """
    testcase.mock_tracker = scaffold.MockTracker()

    testcase.app_class = gracied.Gracie

    testcase.stdout_test = StringIO()
    scaffold.mock("sys.stdout", mock_obj=testcase.stdout_test)

    testcase.test_pidfile_path = "BoGuS_NaMe"
    scaffold.mock(
        "gracied.default_pidfile_path",
        mock_obj=testcase.test_pidfile_path,
        tracker=testcase.mock_tracker)

    scaffold.mock(
        "gracied.OptionParser.error",
        raises=SystemExit,
        tracker=testcase.mock_tracker)

    scaffold.mock(
        "gracied.GracieServer",
        returns=Mock(
            "gracied.GracieServer",
            tracker=testcase.mock_tracker),
        tracker=testcase.mock_tracker)

    scaffold.mock("gracied.default_port", mock_obj=7654)

    testcase.valid_apps = {
        'simple': dict(
            ),
        'argv_loglevel_debug': dict(
            options = ["--log-level", "debug"],
            ),
        'change-pidfile': dict(
            options = ["--pidfile", "/spam/beans.pid"],
            ),
        'change-host': dict(
            options = ["--host", "frobnitz"],
            host = "frobnitz",
            ),
        'change-port': dict(
            options = ["--port", "9779"],
            port = 9779,
            ),
        'change-address': dict(
            options = ["--host", "frobnitz", "--port", "9779"],
            host = "frobnitz",
            port = 9779,
            ),
        'change-root-url': dict(
            options = ["--root-url", "http://spudnik/spam"],
            root_url = "http://spudnik/spam",
            ),
        'change-address-and-root-url': dict(
            options = [
                "--host", "frobnitz", "--port", "9779",
                "--root-url", "http://spudnik/spam",
                ],
            host = "frobnitz",
            port = 9779,
            root_url = "http://frobnitz/spam",
            ),
        }

    for key, params in testcase.valid_apps.items():
        argv = ["progname"]
        options = params.get('options', None)
        if options:
            argv.extend(options)
        params['argv'] = argv
        args = dict(
            argv=argv
            )
        params['args'] = args
        instance = testcase.app_class(**args)
        params['instance'] = instance


class Gracie_TestCase(scaffold.TestCase):
    """ Test cases for Gracie class. """

    def setUp(self):
        """ Set up test fixtures. """
        setup_gracie_fixtures(self)

        scaffold.mock(
            "gracied.Gracie._become_daemon",
            tracker=self.mock_tracker)

    def tearDown(self):
        """ Tear down test fixtures. """
        scaffold.mock_restore()

    def test_instantiate(self):
        """ New Gracie instance should be created. """
        for params in self.valid_apps.values():
            instance = params['instance']
            self.failIfIs(instance, None)

    def test_init_configures_logging(self):
        """ Gracie instance should configure logging. """
        params = self.valid_apps['simple']
        args = params['args']
        logging_prev = gracied.logging
        scaffold.mock(
            "gracied.logging",
            tracker=self.mock_tracker)
        expect_mock_output = """\
           Called gracied.logging.basicConfig(...)
           """
        instance = self.app_class(**args)
        gracied.logging = logging_prev
        self.failUnlessMockCheckerMatch(expect_mock_output)

    def test_wrong_arguments_invokes_parser_error(self):
        """ Wrong number of cmdline arguments should invoke parser error. """
        invalid_argv_params = [
            ["progname", "foo",]
            ]
        expect_error = gracied.OptionParser.error.mock_raises
        expect_mock_output = """\
            Called gracied.OptionParser.error("...")
            """
        for argv in invalid_argv_params:
            args = dict(argv=argv)
            try:
                instance = self.app_class(**args)
            except expect_error:
                pass
            self.failUnlessMockCheckerMatch(expect_mock_output)

    def test_opts_version_performs_version_action(self):
        """ Gracie instance should perform version action. """
        argv = ["progname", "--version"]
        args = dict(argv=argv)
        scaffold.mock(
            "gracied.version", tracker=self.mock_tracker)
        version_test = "Foo.Boo"
        gracied.version.version_full = version_test
        expect_stdout = """\
            ...%(version_test)s...
            """ % vars()
        self.failUnlessRaises(
            SystemExit,
            self.app_class, **args
            )
        self.failUnlessOutputCheckerMatch(
            expect_stdout, self.stdout_test.getvalue()
            )

    def test_opts_help_performs_help_action(self):
        """ Gracie instance should perform help action. """
        argv = ["progname", "--help"]
        args = dict(argv=argv)
        expect_stdout = """\
            Usage: ...
            """
        self.failUnlessRaises(
            SystemExit,
            self.app_class, **args
            )
        self.failUnlessOutputCheckerMatch(
            expect_stdout, self.stdout_test.getvalue()
            )

    def test_opts_loglevel_accepts_specified_value(self):
        """ Gracie instance should accept ‘log-level’ setting. """
        want_loglevel = "DEBUG"
        argv = ["progname", "--log-level", want_loglevel]
        args = dict(argv=argv)
        instance = self.app_class(**args)
        self.failUnlessEqual(want_loglevel, instance.opts.loglevel)

    def test_opts_pidfile_accepts_specified_value(self):
        """ Gracie instance should accept ‘pidfile’ setting. """
        want_path = "/foo/bar.pid"
        argv = ["progname", "--pidfile", want_path]
        args = dict(argv=argv)
        instance = self.app_class(**args)
        self.failUnlessEqual(want_path, instance.opts.pidfile)

    def test_opts_datadir_accepts_specified_value(self):
        """ Gracie instance should accept ‘data-dir’ setting. """
        want_dir = "/foo/bar"
        argv = ["progname", "--data-dir", want_dir]
        args = dict(argv=argv)
        instance = self.app_class(**args)
        self.failUnlessEqual(want_dir, instance.opts.datadir)

    def test_opts_host_accepts_specified_value(self):
        """ Gracie instance should accept ‘host’ setting. """
        want_host = "frobnitz"
        argv = ["progname", "--host", want_host]
        args = dict(argv=argv)
        instance = self.app_class(**args)
        self.failUnlessEqual(want_host, instance.opts.host)

    def test_opts_port_accepts_specified_value(self):
        """ Gracie instance should accept ‘port’ setting. """
        want_port = 9779
        argv = ["progname", "--port", str(want_port)]
        args = dict(argv=argv)
        instance = self.app_class(**args)
        self.failUnlessEqual(want_port, instance.opts.port)

    def test_opts_root_url_accepts_specified_value(self):
        """ Gracie instance should accept ‘root_url’ setting. """
        want_url = "http://spudnik/spam"
        argv = ["progname", "--root-url", want_url]
        args = dict(argv=argv)
        instance = self.app_class(**args)
        self.failUnlessEqual(want_url, instance.opts.root_url)


class Gracie_main_TestCase(scaffold.TestCase):
    """ Test cases for Gracie.main method. """

    def setUp(self):
        """ Set up test fixtures. """
        setup_gracie_fixtures(self)

        scaffold.mock(
            "gracied.Gracie._become_daemon",
            tracker=self.mock_tracker)

    def tearDown(self):
        """ Tear down test fixtures. """
        scaffold.mock_restore()

    def test_instantiates_server(self):
        """ Should create a new server instance. """
        params = self.valid_apps['simple']
        instance = params['instance']
        host = gracied.default_host
        port = gracied.default_port
        expect_mock_output = """\
            Called gracied.GracieServer(
                (%(host)r, %(port)r),
                <Values at ...: {...}>)
            ...""" % vars()
        instance.main()
        self.failUnlessMockCheckerMatch(expect_mock_output)
        self.failIfIs(instance.server, None)

    def test_sets_specified_socket_params(self):
        """ Should set the server on the specified host:port. """
        for params in self.valid_apps.values():
            instance = params['instance']
            default_address = (
                gracied.default_host, gracied.default_port
                )
            host = params.get('host', gracied.default_host)
            port = params.get('port', gracied.default_port)
            if (host, port) == default_address:
                continue
            expect_mock_output = """\
                Called gracied.GracieServer(
                    (%(host)r, %(port)r),
                    <Values at ...: {...}>)
                ...""" % vars()
            instance.main()
            self.failUnlessMockCheckerMatch(expect_mock_output)
            self.mock_tracker.clear()

    def test_becomes_daemon(self):
        """ Should attempt to become a daemon. """
        params = self.valid_apps['simple']
        instance = params['instance']
        expect_mock_output = """\
            ...
            Called gracied.Gracie._become_daemon()
            ...
            """
        instance.main()
        self.failUnlessMockCheckerMatch(expect_mock_output)

    def test_starts_server(self):
        """ Should start GracieServer if child fork. """
        params = self.valid_apps['simple']
        instance = params['instance']
        port = gracied.default_port
        expect_mock_output = """\
            ...
            Called gracied.GracieServer.serve_forever()
            """
        instance.main()
        self.failUnlessMockCheckerMatch(expect_mock_output)


class Gracie_become_daemon_TestCase(scaffold.TestCase):
    """ Test cases for Gracie._become_daemon method. """

    def setUp(self):
        """ Set up test fixtures. """
        setup_gracie_fixtures(self)

        self.mock_context = scaffold.Mock(
            "DaemonContext",
            tracker=self.mock_tracker)

        # Ensure that using this object as a context manager will call
        # the appropriate method.
        self.mock_context.__enter__.mock_returns_func = (
            self.mock_context.open)

        scaffold.mock(
            "daemon.DaemonContext",
            returns=self.mock_context,
            tracker=self.mock_tracker)

        self.test_working_directory = "BoGuS_DiR"
        scaffold.mock(
            "os.getcwd",
            returns=self.test_working_directory,
            tracker=self.mock_tracker)

        self.mock_lockfile = scaffold.Mock(
            "PIDLockFile",
            tracker=self.mock_tracker)

        scaffold.mock(
            "pidlockfile.PIDLockFile",
            returns=self.mock_lockfile,
            tracker=self.mock_tracker)

    def tearDown(self):
        """ Tear down test fixtures. """
        scaffold.mock_restore()

    def test_creates_pidlockfile(self):
        """ Should create a PIDLockFile. """
        params = self.valid_apps['simple']
        instance = params['instance']
        pidfile_path = os.path.join(
            self.test_working_directory, self.test_pidfile_path)
        expect_mock_output = """\
            ...
            Called pidlockfile.PIDLockFile(%(pidfile_path)r)
            ...""" % vars()
        instance._become_daemon()
        self.failUnlessMockCheckerMatch(expect_mock_output)

    def test_creates_daemon_context(self):
        """ Should create a DaemonContext instance with expected args. """
        params = self.valid_apps['simple']
        instance = params['instance']
        expect_pidfile = self.mock_lockfile
        expect_files_preserve = [
            sys.stderr,
            ]
        expect_mock_output = """\
            ...
            Called daemon.DaemonContext(
                files_preserve=%(expect_files_preserve)r,
                pidfile=%(expect_pidfile)r)
            ...""" % vars()
        instance._become_daemon()
        scaffold.mock_restore()
        self.failUnlessMockCheckerMatch(expect_mock_output)

    def test_invokes_daemon_context_open(self):
        """ Should invoke DaemonContext open method. """
        params = self.valid_apps['simple']
        instance = params['instance']
        expect_mock_output = """\
            ...
            Called DaemonContext.open()
            """
        instance._become_daemon()
        self.failUnlessMockCheckerMatch(expect_mock_output)

    def test_stores_daemon_context(self):
        """ Should store daemon context as attribute of app. """
        params = self.valid_apps['simple']
        instance = params['instance']
        expect_context = self.mock_context
        instance._become_daemon()
        self.failUnlessIs(expect_context, instance.daemon_context)


class ProgramMain_TestCase(scaffold.ProgramMain_TestCase):
    """ Test cases for program __main__ function. """

    def setUp(self):
        """ Set up test fixtures """
        self.program_module = gracied
        self.application_class = gracied.Gracie
        super(ProgramMain_TestCase, self).setUp()



# -*- coding: utf-8 -*-

# gracie/version/__init__.py
# Part of Gracie, an OpenID provider.
#
# Copyright © 2008–2009 Ben Finney <ben+python@benfinney.id.au>
# This is free software; you may copy, modify and/or distribute this work
# under the terms of the GNU General Public License, version 2 or later.
# No warranty expressed or implied. See the file ‘LICENSE.GPL-2’ for details.

""" Version information for the Gracie package. """

from version_info import version_info

version_info['version_string'] = "0.2.10"

version_short = "%(version_string)s" % version_info
version_full = "%(version_string)s.r%(revno)s" % version_info
version = version_short

author_name = "Ben Finney"
author_email = "ben+python@benfinney.id.au"
author = "%(author_name)s <%(author_email)s>" % vars()

copyright_year_begin = "2007"
date = version_info['date'].split(' ', 1)[0]
copyright_year = date.split('-')[0]
copyright_year_range = copyright_year_begin
if copyright_year > copyright_year_begin:
    copyright_year_range += "–%(copyright_year)s" % vars()

copyright = "Copyright © %(copyright_year_range)s %(author)s" % vars()
license = "GPL-2+"



# -*- coding: utf-8 -*-

# test/scaffold.py
#
# Copyright © 2007–2009 Ben Finney <ben+python@benfinney.id.au>
# This is free software; you may copy, modify and/or distribute this work
# under the terms of the GNU General Public License, version 2 or later.
# No warranty expressed or implied. See the file ‘LICENSE.GPL-2’ for details.

""" Scaffolding for unit test modules.
    """

import unittest
import doctest
import logging
import os
import sys
import operator
import textwrap
from minimock import (
    Mock,
    TraceTracker as MockTracker,
    mock,
    restore as mock_restore,
    )

test_dir = os.path.dirname(os.path.abspath(__file__))
parent_dir = os.path.dirname(test_dir)
if not test_dir in sys.path:
    sys.path.insert(1, test_dir)
if not parent_dir in sys.path:
    sys.path.insert(1, parent_dir)
bin_dir = os.path.join(parent_dir, "bin")

# Disable all but the most critical logging messages
logging.disable(logging.CRITICAL)


def get_python_module_names(file_list, file_suffix='.py'):
    """ Return a list of module names from a filename list. """
    module_names = [m[:m.rfind(file_suffix)] for m in file_list
        if m.endswith(file_suffix)]
    return module_names


def get_test_module_names(module_list, module_prefix='test_'):
    """ Return the list of module names that qualify as test modules. """
    module_names = [m for m in module_list
        if m.startswith(module_prefix)]
    return module_names


def make_suite(path=test_dir):
    """ Create the test suite for the given path. """
    loader = unittest.TestLoader()
    python_module_names = get_python_module_names(os.listdir(path))
    test_module_names = get_test_module_names(python_module_names)
    suite = loader.loadTestsFromNames(test_module_names)

    return suite


def unittest_main(argv=None):
    """ Mainline function for each unit test module. """

    from sys import argv as sys_argv
    if not argv:
        argv = sys_argv

    exitcode = None
    try:
        unittest.main(argv=argv, defaultTest='suite')
    except SystemExit, e:
        exitcode = e.code

    return exitcode


def make_module_from_file(module_name, file_name):
    """ Make a new module object from the code in specified file. """

    from types import ModuleType
    module = ModuleType(module_name)

    module_file = open(file_name, 'r')
    exec module_file in module.__dict__

    return module


class TestCase(unittest.TestCase):
    """ Test case behaviour. """

    def failUnlessRaises(self, exc_class, func, *args, **kwargs):
        """ Fail unless the function call raises the expected exception.

            Fail the test if an instance of the exception class
            ``exc_class`` is not raised when calling ``func`` with the
            arguments ``*args`` and ``**kwargs``.

            """
        try:
            super(TestCase, self).failUnlessRaises(
                exc_class, func, *args, **kwargs)
        except self.failureException:
            exc_class_name = exc_class.__name__
            msg = (
                "Exception %(exc_class_name)s not raised"
                " for function call:"
                " func=%(func)r args=%(args)r kwargs=%(kwargs)r"
                ) % vars()
            raise self.failureException(msg)

    def failIfIs(self, first, second, msg=None):
        """ Fail if the two objects are identical.

            Fail the test if ``first`` and ``second`` are identical,
            as determined by the ``is`` operator.

            """
        if first is second:
            if msg is None:
                msg = "%(first)r is %(second)r" % vars()
            raise self.failureException(msg)

    def failUnlessIs(self, first, second, msg=None):
        """ Fail unless the two objects are identical.

            Fail the test unless ``first`` and ``second`` are
            identical, as determined by the ``is`` operator.

            """
        if first is not second:
            if msg is None:
                msg = "%(first)r is not %(second)r" % vars()
            raise self.failureException(msg)

    assertIs = failUnlessIs
    assertNotIs = failIfIs

    def failIfIn(self, first, second, msg=None):
        """ Fail if the second object is in the first.

            Fail the test if ``first`` contains ``second``, as
            determined by the ``in`` operator.

            """
        if second in first:
            if msg is None:
                msg = "%(second)r is in %(first)r" % vars()
            raise self.failureException(msg)

    def failUnlessIn(self, first, second, msg=None):
        """ Fail unless the second object is in the first.

            Fail the test unless ``first`` contains ``second``, as
            determined by the ``in`` operator.

            """
        if second not in first:
            if msg is None:
                msg = "%(second)r is not in %(first)r" % vars()
            raise self.failureException(msg)

    assertIn = failUnlessIn
    assertNotIn = failIfIn

    def failUnlessOutputCheckerMatch(self, want, got, msg=None):
        """ Fail unless the specified string matches the expected.

            Fail the test unless ``want`` matches ``got``, as
            determined by a ``doctest.OutputChecker`` instance. This
            is not an equality check, but a pattern match according to
            the ``OutputChecker`` rules.

            """
        checker = doctest.OutputChecker()
        want = textwrap.dedent(want)
        source = ""
        example = doctest.Example(source, want)
        got = textwrap.dedent(got)
        checker_optionflags = reduce(operator.or_, [
            doctest.ELLIPSIS,
            ])
        if not checker.check_output(want, got, checker_optionflags):
            if msg is None:
                diff = checker.output_difference(
                    example, got, checker_optionflags)
                msg = "\n".join([
                    "Output received did not match expected output",
                    "%(diff)s",
                    ]) % vars()
            raise self.failureException(msg)

    assertOutputCheckerMatch = failUnlessOutputCheckerMatch

    def failUnlessMockCheckerMatch(self, want, tracker=None, msg=None):
        """ Fail unless the mock tracker matches the wanted output.

            Fail the test unless `want` matches the output tracked by
            `tracker` (defaults to ``self.mock_tracker``. This is not
            an equality check, but a pattern match according to the
            ``minimock.MinimockOutputChecker`` rules.

            """
        if tracker is None:
            tracker = self.mock_tracker
        if not tracker.check(want):
            if msg is None:
                diff = tracker.diff(want)
                msg = "\n".join([
                    "Output received did not match expected output",
                    "%(diff)s",
                    ]) % vars()
            raise self.failureException(msg)

    def failIfMockCheckerMatch(self, want, tracker=None, msg=None):
        """ Fail if the mock tracker matches the specified output.

            Fail the test if `want` matches the output tracked by
            `tracker` (defaults to ``self.mock_tracker``. This is not
            an equality check, but a pattern match according to the
            ``minimock.MinimockOutputChecker`` rules.

            """
        if tracker is None:
            tracker = self.mock_tracker
        if tracker.check(want):
            if msg is None:
                diff = tracker.diff(want)
                msg = "\n".join([
                    "Output received matched specified undesired output",
                    "%(diff)s",
                    ]) % vars()
            raise self.failureException(msg)

    assertMockCheckerMatch = failUnlessMockCheckerMatch
    assertNotMockCheckerMatch = failIfMockCheckerMatch

    def failIfIsInstance(self, obj, classes, msg=None):
        """ Fail if the object is an instance of the specified classes.

            Fail the test if the object ``obj`` is an instance of any
            of ``classes``.

            """
        if isinstance(obj, classes):
            if msg is None:
                msg = (
                    "%(obj)r is an instance of one of %(classes)r"
                    ) % vars()
            raise self.failureException(msg)

    def failUnlessIsInstance(self, obj, classes, msg=None):
        """ Fail unless the object is an instance of the specified classes.

            Fail the test unless the object ``obj`` is an instance of
            any of ``classes``.

            """
        if not isinstance(obj, classes):
            if msg is None:
                msg = (
                    "%(obj)r is not an instance of any of %(classes)r"
                    ) % vars()
            raise self.failureException(msg)

    assertIsInstance = failUnlessIsInstance
    assertNotIsInstance = failIfIsInstance

    def failUnlessFunctionInTraceback(self, traceback, function, msg=None):
        """ Fail if the function is not in the traceback.

            Fail the test if the function ``function`` is not at any
            of the levels in the traceback object ``traceback``.

            """
        func_in_traceback = False
        expect_code = function.func_code
        current_traceback = traceback
        while current_traceback is not None:
            if expect_code is current_traceback.tb_frame.f_code:
                func_in_traceback = True
                break
            current_traceback = current_traceback.tb_next

        if not func_in_traceback:
            if msg is None:
                msg = (
                    "Traceback did not lead to original function"
                    " %(function)s"
                    ) % vars()
            raise self.failureException(msg)

    assertFunctionInTraceback = failUnlessFunctionInTraceback


class Exception_TestCase(TestCase):
    """ Test cases for exception classes. """

    def __init__(self, *args, **kwargs):
        """ Set up a new instance """
        self.valid_exceptions = NotImplemented
        super(Exception_TestCase, self).__init__(*args, **kwargs)

    def setUp(self):
        """ Set up test fixtures. """
        for exc_type, params in self.valid_exceptions.items():
            args = (None, ) * params['min_args']
            params['args'] = args
            instance = exc_type(*args)
            params['instance'] = instance

        super(Exception_TestCase, self).setUp()

    def test_exception_instance(self):
        """ Exception instance should be created. """
        for params in self.valid_exceptions.values():
            instance = params['instance']
            self.failIfIs(None, instance)

    def test_exception_types(self):
        """ Exception instances should match expected types. """
        for params in self.valid_exceptions.values():
            instance = params['instance']
            for match_type in params['types']:
                match_type_name = match_type.__name__
                fail_msg = (
                    "%(instance)r is not an instance of"
                    " %(match_type_name)s"
                    ) % vars()
                self.failUnless(
                    isinstance(instance, match_type),
                    msg=fail_msg)


class ProgramMain_TestCase(TestCase):
    """ Test cases for program __main__ function.

        Tests a module-level function named __main__ with behaviour
        inspired by Guido van Rossum's post "Python main() functions"
        <URL:http://www.artima.com/weblogs/viewpost.jsp?thread=4829>.

        It expects:
          * the program module has a __main__ function, that:
              * accepts an 'argv' argument, defaulting to sys.argv
              * instantiates a program application class, passing argv
              * calls the application's main() method with no arguments
              * catches SystemExit and returns the error code
          * the application behaviour is defined in a class, that:
              * has an __init__() method accepting an 'argv' argument as
                the commandline argument list to parse
              * has a main() method responsible for running the program,
                and returning on successful program completion
              * raises SystemExit when an abnormal exit is required

        """

    def __init__(self, *args, **kwargs):
        """ Set up a new instance. """
        self.program_module = NotImplemented
        self.application_class = NotImplemented
        super(ProgramMain_TestCase, self).__init__(*args, **kwargs)

    def setUp(self):
        """ Set up test fixtures. """
        self.mock_tracker = MockTracker()

        self.app_class_name = self.application_class.__name__
        self.mock_app = Mock("test_app", tracker=self.mock_tracker)
        self.mock_app_class = Mock(self.app_class_name,
            tracker=self.mock_tracker)
        self.mock_app_class.mock_returns = self.mock_app
        mock(self.app_class_name, mock_obj=self.mock_app_class,
            nsdicts=[self.program_module.__dict__])

        super(ProgramMain_TestCase, self).setUp()

    def tearDown(self):
        """ Tear down test fixtures. """
        mock_restore()
        super(ProgramMain_TestCase, self).tearDown()

    def test_main_should_instantiate_app(self):
        """ __main__() should instantiate application class. """
        app_class_name = self.app_class_name
        argv = ["foo", "bar"]
        expect_mock_output = """\
            Called %(app_class_name)s(%(argv)r)...
            """ % vars()
        self.program_module.__main__(argv)
        self.failUnlessMockCheckerMatch(expect_mock_output)

    def test_main_should_call_app_main(self):
        """ __main__() should call the application main method. """
        argv = ["foo", "bar"]
        app_class_name = self.app_class_name
        expect_mock_output = """\
            Called %(app_class_name)s(%(argv)r)
            Called test_app.main()
            """ % vars()
        self.program_module.__main__(argv)
        self.failUnlessMockCheckerMatch(expect_mock_output)

    def test_main_no_argv_should_supply_sys_argv(self):
        """ __main__() with no argv should supply sys.argv to application. """
        sys_argv_test = ["foo", "bar"]
        mock("sys.argv", mock_obj=sys_argv_test)
        app_class_name = self.app_class_name
        expect_mock_output = """\
            Called %(app_class_name)s(%(sys_argv_test)r)
            Called test_app.main()
            """ % vars()
        self.program_module.__main__()
        self.failUnlessMockCheckerMatch(expect_mock_output)

    def test_main_should_return_none_on_success(self):
        """ __main__() should return None when no SystemExit raised. """
        expect_exit_code = None
        exit_code = self.program_module.__main__()
        self.failUnlessEqual(expect_exit_code, exit_code)

    def test_main_should_return_exit_code_on_system_exit(self):
        """ __main__() should return application SystemExit code. """
        expect_exit_code = object()
        self.mock_app.main.mock_raises = SystemExit(expect_exit_code)
        exit_code = self.program_module.__main__()
        self.failUnlessEqual(expect_exit_code, exit_code)



#! /usr/bin/env python
# -*- coding: utf-8 -*-

# setup.py
# Part of Gracie, an OpenID provider.
#
# Copyright © 2007–2009 Ben Finney <ben+python@benfinney.id.au>
# This is free software; you may copy, modify and/or distribute this work
# under the terms of the GNU General Public License, version 2 or later.
# No warranty expressed or implied. See the file ‘LICENSE.GPL-2’ for details.

""" Package setup script.
    """

import distutils.cmd
import distutils.command.build
import distutils.command.install
import distutils.command.clean
import distutils.log
import distutils.util
import os
import os.path
import glob
import errno
import subprocess
import re
import textwrap
import itertools

from setuptools import setup, find_packages
import docutils.core

distribution_name = "gracie"
main_module_name = 'gracie'
main_module = __import__(main_module_name, fromlist=['version'])
version = main_module.version

short_description, long_description = (
    textwrap.dedent(desc).strip()
    for desc in main_module.__doc__.split('\n\n', 1)
    )


def is_source_file_newer(source_path, destination_path, force=False):
    """ Return True if destination is older than source or does not exist. """
    if force:
        result = True
    else:
        source_stat = os.stat(source_path)
        source_ctime = source_stat.st_ctime
        try:
            destination_stat = os.stat(destination_path)
        except OSError, exc:
            if exc.errno == errno.ENOENT:
                destination_ctime = None
            else:
                raise
        else:
            destination_ctime = destination_stat.st_ctime
        result = (source_ctime > destination_ctime)

    return result


class BuildDocumentationCommand(distutils.cmd.Command):
    """ Build documentation for this distribution. """
    user_options = [
        ("force", 'f',
         "Forcibly build everything (ignore file timestamps)."),
        ("html-src-files=", None,
         "Source files to build to HTML documents."),
        ("manpage-8-src-files=", None,
         "Source files to build to Unix manpages for section 8."),
        ("logo-src-file=", None,
         "Source SVG document for project logo."),
        ]

    boolean_options = ['force']

    def initialize_options(self):
        """ Initialise command options to defaults. """
        self.document_transforms = {
            'html': {
                'func': render_rst_document,
                'source_name_option': 'html_src_files',
                'writer_name': 'html',
                'source_suffix_regex': re.compile("\.txt$"),
                'dest_suffix': ".html",
                },
            'manpage.8': {
                'func': render_rst_document,
                'source_name_option': 'manpage_8_src_files',
                'writer_name': 'manpage',
                'source_suffix_regex': re.compile("\.8\.txt$"),
                'dest_suffix': ".8",
                },
            'logo': {
                'func': render_svg_document,
                'source_name_option': 'logo_src_file',
                'size': 80,
                'format': 'PNG',
                'source_suffix_regex': re.compile("\.svg$"),
                'dest_suffix': ".png",
                },
            }

        self.force = None

        for transform in self.document_transforms.values():
            option_name = transform['source_name_option']
            setattr(self, option_name, None)

    def finalize_options(self):
        """ Finalise command options before execution. """
        self.set_undefined_options(
            'build',
            ('force', 'force'))

        for (transform_name, transform) in self.document_transforms.items():
            source_paths = []
            option_name = transform['source_name_option']
            source_files_option_value = getattr(self, option_name, None)
            if source_files_option_value is not None:
                source_paths = source_files_option_value.split()
            transform['source_paths'] = source_paths

    def _render_documents(self, transform):
        """ Render documents that are not up-to-date. """
        for in_file_path in transform['source_paths']:
            out_file_base = re.sub(
                transform['source_suffix_regex'], "",
                in_file_path)
            out_file_path = out_file_base + transform['dest_suffix']
            render_document_func = transform['func']
            if is_source_file_newer(in_file_path, out_file_path, self.force):
                out_file_dir = os.path.dirname(out_file_path)
                if not os.path.isdir(out_file_dir):
                    self.mkpath(out_file_dir)
                render_document_func(in_file_path, out_file_path, transform)

    def run(self):
        """ Execute this command. """
        for transform in self.document_transforms.values():
            self._render_documents(transform)


class BuildCommand(distutils.command.build.build):
    """ Custom ‘build’ command for this distribution. """

    sub_commands = (
        [('build_doc', lambda self: True)]
        + distutils.command.build.build.sub_commands)


class InstallCommand(distutils.command.install.install):
    """ Custom ‘install’ command for this distribution. """

    sub_commands = (
        [('build_doc', lambda self: True)]
        + distutils.command.install.install.sub_commands)


def render_rst_document(in_file_path, out_file_path, transform):
    """ Render a document from source to dest using specified writer. """
    in_file_path = distutils.util.convert_path(in_file_path)
    out_file_path = distutils.util.convert_path(out_file_path)
    writer = transform['writer_name']
    distutils.log.info(
        "using writer %(writer)r to render document"
        " %(in_file_path)r -> %(out_file_path)r"
        % vars())
    docutils.core.publish_file(
        source_path=in_file_path,
        destination_path=out_file_path,
        writer_name=transform['writer_name'])


def render_svg_document(in_file_path, out_file_path, transform):
    """ Render SVG document to destination logo file. """
    in_file_path = distutils.util.convert_path(in_file_path)
    out_file_path = distutils.util.convert_path(out_file_path)
    geometry = "%(size)dx%(size)d" % transform
    render_process_args = [
        "gm", "convert",
        "-format", "SVG", in_file_path,
        "-geometry", geometry,
        "-format", transform['format'], out_file_path,
        ]
    distutils.log.info(
        "rendering logo %(in_file_path)r -> %(out_file_path)r"
        % vars())
    subprocess.call(render_process_args)


class CleanDocumentationCommand(distutils.cmd.Command):
    """ Clean files generated for this distribution's documentation. """
    user_options = [
        ("all", 'a',
         "Remove all build output, not just temporary by-products."),
        ("generated-files=", None,
         "File globs of generated documentation files."),
        ]

    boolean_options = ['all']

    def initialize_options(self):
        """ Initialise command options to defaults. """
        self.all = None

        self.generated_files = None

    def finalize_options(self):
        """ Finalise command options before execution. """
        self.set_undefined_options(
            'clean',
            ('all', 'all'))

        self.generated_file_globs = self.generated_files.split()

    def run(self):
        """ Execute this command. """
        generated_file_paths = set()
        for file_glob in self.generated_file_globs:
            generated_file_paths.update(set(
                glob.glob(file_glob)))
        for file_path in generated_file_paths:
            os.remove(file_path)


class CleanCommand(distutils.command.clean.clean, object):
    """ Custom ‘clean’ command for this distribution. """

    sub_commands = (
        [('clean_doc', lambda self: True)]
        + distutils.command.clean.clean.sub_commands)

    def run(self):
        for cmd_name in self.get_sub_commands():
            self.run_command(cmd_name)
        super(CleanCommand, self).run()


setup(
    name=distribution_name,
    version=version.version,
    packages=find_packages(
        exclude=['test'],
        ),
    scripts=[
        "bin/gracied",
        ],
    cmdclass={
        "build": BuildCommand,
        "build_doc": BuildDocumentationCommand,
        "install": InstallCommand,
        "clean": CleanCommand,
        "clean_doc": CleanDocumentationCommand,
        },

    # setuptools metadata
    zip_safe=False,
    test_suite="test.suite",
    install_requires=[
        "setuptools",
        "python-daemon >= 1.4.5",
        "python-openid >= 1.2",
        "Routes >= 1.6.3",
        ],
    tests_require=[
        "MiniMock >= 1.2.2",
        ],

    # PyPI metadata
    author=version.author_name,
    author_email=version.author_email,
    description=short_description,
    license=version.license,
    keywords=[
        "gracie", "openid", "identity", "authentication", "provider",
        ],
    url=main_module._url,
    long_description=long_description,
    classifiers=[
        # Reference: http://pypi.python.org/pypi?%3Aaction=list_classifiers
        "Development Status :: 3 - Alpha",
        "License :: OSI Approved :: GNU General Public License (GPL)",
        "Programming Language :: Python",
        "Topic :: Internet",
        "Topic :: System",
        "Operating System :: POSIX",
        "Intended Audience :: System Administrators",
        ],
    )


# Local variables:
# mode: python
# coding: utf-8
# End:
# vim: filetype=python fileencoding=utf-8 :



# -*- coding: utf-8 -*-

# test/test_httprequest.py
# Part of Gracie, an OpenID provider.
#
# Copyright © 2007–2009 Ben Finney <ben+python@benfinney.id.au>
# This is free software; you may copy, modify and/or distribute this work
# under the terms of the GNU General Public License, version 2 or later.
# No warranty expressed or implied. See the file ‘LICENSE.GPL-2’ for details.

""" Unit test for httprequest module.
    """

import sys
from StringIO import StringIO
import logging
import urllib

import scaffold
from scaffold import Mock
from test_authservice import Stub_AuthService
from test_server import (
    Stub_OpenIDStore, Stub_OpenIDServer, Stub_OpenIDError,
    Stub_OpenIDRequest, Stub_OpenIDResponse, Stub_OpenIDWebResponse,
    )

from gracie import httprequest
from gracie import pagetemplate


class Stub_Logger(object):
    """ Stub class for Logger """

    def log(self, format, *args, **kwargs):
        """ Log a message """

    error = log
    warn = log
    info = log
    debug = log

class Stub_SessionManager(object):
    """ Stub class for SessionManager """

    def __init__(self):
        """ Set up a new instance """
        self._sessions = dict()

    def create_session(self, session):
        username = session.get('username')
        session_id = session.get('session_id')
        if session_id is None:
            if username is None:
                session_id = "DEADBEEF"
            else:
                session_id = "DEADBEEF-%(username)s" % session
        session.update(dict(session_id = session_id))
        self._sessions[session_id] = session
        return session_id

    def get_session(self, session_id):
        return self._sessions[session_id]

    def remove_session(self, session_id):
        del self._sessions[session_id]

class Stub_HTTPServer(object):
    """ Stub class for HTTPServer """

    def __init__(self, server_address, handler_class, gracie_server):
        """ Set up a new instance """
        self.gracie_server = gracie_server
        (host, port) = server_address
        self.server_location = "%(host)s:%(port)s" % vars()

class Stub_HTTPRequestHandler(object):
    """ Stub class for HTTPRequestHandler """

class Stub_OptionValues(object):
    """ Stub class for optparse.OptionValues """

    def __init__(self, opt_dict):
        """ Set up a new instance """
        for key, value in opt_dict.items():
            setattr(self, key, value)

class Stub_GracieServer(object):
    """ Stub class for GracieServer """

    version = "3.14.test"

    def __init__(self, opts):
        """ Set up a new instance """
        self.opts = opts
        server_address = (opts.host, opts.port)
        self.server_location = "%s:%s" % server_address
        self.http_server = Stub_HTTPServer(
            server_address, Stub_HTTPRequestHandler(), self
            )
        store = Stub_OpenIDStore(None)
        self.openid_server = Stub_OpenIDServer(store)
        self.auth_service = Stub_AuthService()
        self.sess_manager = Stub_SessionManager()


class Stub_TCPConnection(object):
    """ Stub class for TCP connection """

    def __init__(self, text):
        """ Set up a new instance """
        self._text = text

    def makefile(self, mode, bufsize):
        """ Make a file handle to the connection stream """
        conn_file = None
        if mode.startswith('r'):
            conn_file = StringIO(self._text)
            conn_file.seek(0)
        elif mode.startswith('w'):
            conn_file = StringIO("")
        return conn_file

class Stub_Request(object):
    """ Stub class for HTTP request encapsulation """

    def __init__(
        self, method, path,
        version="HTTP/1.1", header=None, query=None,
        ):
        """ Set up a new instance """
        self.method = method.upper()
        self.path = path
        self.version = version
        if header is None:
            header = []
        self.header = header
        if query is None:
            query = dict()
        self.query = query
        self.data = ""

    def _encode_query(self):
        if self.query:
            query_text = urllib.urlencode(self.query)

            if self.method in ["POST", "PUT"]:
                self.header.append(
                    ("Content-Length", str(len(query_text)))
                    )
                self.data += query_text
            elif self.method in ["GET", "HEAD"]:
                path = self.path
                self.path = "%(path)s?%(query_text)s" % vars()

    def __str__(self):
        self._encode_query()
        command_text = "%(method)s %(path)s %(version)s" % vars(self)
        header_text = "\n".join(
            "%s: %s" % (name, val)
            for name, val in self.header)

        lines = []
        lines.append(command_text)
        if header_text:
            lines.append(header_text)
        lines.append("")
        if self.data:
            lines.append(self.data)

        text = "\n".join(lines)
        return text

    def connection(self):
        """ Return a TCP connection to this request """
        return Stub_TCPConnection(str(self))


class HTTPRequestHandler_TestCase(scaffold.TestCase):
    """ Test cases for HTTPRequestHandler class. """

    def setUp(self):
        """ Set up test fixtures """
        self.mock_tracker = scaffold.MockTracker()

        self.handler_class = httprequest.HTTPRequestHandler

        self.stdout_test = StringIO()

        scaffold.mock(
            "httprequest.Response",
            returns=Mock(
                "httprequest.Response",
                tracker=self.mock_tracker),
            tracker=self.mock_tracker)
        scaffold.mock(
            "httprequest.ResponseHeader",
            returns=Mock(
                "httprequest.ResponseHeader",
                tracker=self.mock_tracker),
            tracker=self.mock_tracker)
        scaffold.mock(
            "pagetemplate.Page",
            returns=Mock(
                "pagetemplate.Page",
                tracker=self.mock_tracker),
            tracker=self.mock_tracker)
        scaffold.mock(
            "httprequest.session_cookie_name",
            mock_obj="TEST_session")

        self.valid_requests = {
            'get-bogus': dict(
                request = Stub_Request("GET", "/bogus"),
                ),
            'get-root': dict(
                request = Stub_Request("GET", "/"),
                ),
            'no-cookie': dict(
                request = Stub_Request("GET", "/"),
                ),
            'unknown-cookie': dict(
                request = Stub_Request("GET", "/",
                    header = [
                        ("Cookie", "TEST_session=DECAFBAD"),
                        ],
                    ),
                session_id = "DECAFBAD",
                ),
            'good-cookie': dict(
                identity_name = "fred",
                request = Stub_Request("GET", "/",
                    header = [
                        ("Cookie", "TEST_session=DEADBEEF-fred"),
                        ],
                    ),
                session = dict(
                    session_id = "DEADBEEF-fred",
                    username = "fred",
                    ),
                ),
            'id-bogus': dict(
                identity_name = "bogus",
                request = Stub_Request("GET", "/id/bogus"),
                ),
            'id-fred': dict(
                identity_name = "fred",
                request = Stub_Request("GET", "/id/fred"),
                ),
            'logout': dict(
                request = Stub_Request("GET", "/logout"),
                ),
            'login': dict(
                request = Stub_Request("GET", "/login"),
                ),
            'nobutton-login': dict(
                request = Stub_Request("POST", "/login",
                    query = dict(
                        username="bogus",
                        password="bogus",
                        ),
                    ),
                ),
            'cancel-login': dict(
                request = Stub_Request("POST", "/login",
                    query = dict(
                        username="bogus",
                        password="bogus",
                        cancel="Cancel",
                        ),
                    ),
                ),
            'login-bogus': dict(
                identity_name = "bogus",
                request = Stub_Request("POST", "/login",
                    query = dict(
                        username="bogus",
                        password="bogus",
                        submit="Sign in",
                        ),
                    ),
                ),
            'login-fred-wrong': dict(
                identity_name = "fred",
                request = Stub_Request("POST", "/login",
                    query = dict(
                        username="fred",
                        password="password23",
                        submit="Sign in",
                        ),
                    ),
                ),
            'login-fred-okay': dict(
                identity_name = "fred",
                request = Stub_Request("POST", "/login",
                    query = dict(
                        username="fred",
                        password="password1",
                        submit="Sign in",
                        ),
                    ),
                ),
            'openid-no-query': dict(
                request = Stub_Request("GET", "/openidserver"),
                ),
            'openid-bogus-query': dict(
                request = Stub_Request("GET", "/openidserver",
                    query = {
                        "foo.bar": "spam",
                        "flim.flam": "",
                        "wibble.wobble": "eggs",
                        },
                    ),
                ),
            'openid-query-associate': dict(
                request = Stub_Request("GET", "/openidserver",
                    query = {
                        "openid.mode": "associate",
                        "openid.session_type": "",
                        },
                    ),
                ),
            'openid-query-checkid_immediate-no-session': dict(
                request = Stub_Request("GET", "/openidserver",
                    header = [],
                    query = {
                        "openid.mode": "checkid_immediate",
                        "openid.identity": "http://example.org:0/id/fred",
                        "openid.return_to": "http://example.com/",
                        },
                    ),
                ),
            'openid-query-checkid_setup-no-session': dict(
                request = Stub_Request("GET", "/openidserver",
                    header = [],
                    query = {
                        "openid.mode": "checkid_setup",
                        "openid.identity": "http://example.org:0/id/fred",
                        "openid.return_to": "http://example.com/",
                        },
                    ),
                ),
            'openid-query-checkid_immediate-other-session': dict(
                request = Stub_Request("GET", "/openidserver",
                    header = [
                        ("Cookie", "TEST_session=DEADBEEF-bill"),
                        ],
                    query = {
                        "openid.mode": "checkid_immediate",
                        "openid.identity": "http://example.org:0/id/fred",
                        "openid.return_to": "http://example.com/",
                        },
                    ),
                session = dict(
                    session_id = "DEADBEEF-bill",
                    username = "bill",
                    ),
                ),
            'openid-query-checkid_setup-other-session': dict(
                request = Stub_Request("GET", "/openidserver",
                    header = [
                        ("Cookie", "TEST_session=DEADBEEF-bill"),
                        ],
                    query = {
                        "openid.mode": "checkid_setup",
                        "openid.identity": "http://example.org:0/id/fred",
                        "openid.return_to": "http://example.com/",
                        },
                    ),
                session = dict(
                    session_id = "DEADBEEF-bill",
                    username = "bill",
                    ),
                ),
            'openid-query-checkid_immediate-right-session': dict(
                request = Stub_Request("GET", "/openidserver",
                    header = [
                        ("Cookie", "TEST_session=DEADBEEF-fred"),
                        ],
                    query = {
                        "openid.mode": "checkid_immediate",
                        "openid.identity": "http://example.org:0/id/fred",
                        "openid.return_to": "http://example.com/",
                        },
                    ),
                session = dict(
                    session_id = "DEADBEEF-fred",
                    username = "fred",
                    ),
                ),
            'openid-query-checkid_setup-right-session': dict(
                request = Stub_Request("GET", "/openidserver",
                    header = [
                        ("Cookie", "TEST_session=DEADBEEF-fred"),
                        ],
                    query = {
                        "openid.mode": "checkid_setup",
                        "openid.identity": "http://example.org:0/id/fred",
                        "openid.return_to": "http://example.com/",
                        },
                    ),
                session = dict(
                    session_id = "DEADBEEF-fred",
                    username = "fred",
                    ),
                ),
            'openid-cancel-login': dict(
                request = Stub_Request("POST", "/login",
                    header = [
                        ("Cookie", "TEST_session=DEADBEEF"),
                        ],
                    query = dict(
                        username = "bogus",
                        password = "bogus",
                        cancel = "Cancel",
                        ),
                    ),
                session = dict(
                    session_id = "DEADBEEF",
                    last_openid_request = Stub_OpenIDRequest(
                        http_query = {'openid.mode': "checkid_setup"},
                        params = dict(
                            identity = "http://example.org:0/id/fred",
                            trust_root = "http://example.com/",
                            return_to = "http://example.com/account",
                            ),
                        ),
                    ),
                ),
            'openid-login-bill-other': dict(
                identity_name = "fred",
                request = Stub_Request("POST", "/login",
                    header = [
                        ("Cookie", "TEST_session=DEADBEEF"),
                        ],
                    query = dict(
                        username="bill",
                        password="secret1",
                        submit="Sign in",
                        ),
                    ),
                session = dict(
                    session_id = "DEADBEEF",
                    last_openid_request = Stub_OpenIDRequest(
                        http_query = {'openid.mode': "checkid_setup"},
                        params = dict(
                            identity = "http://example.org:0/id/fred",
                            trust_root = "http://example.com/",
                            return_to = "http://example.com/account",
                            ),
                        ),
                    ),
                ),
            'openid-login-fred-okay': dict(
                identity_name = "fred",
                request = Stub_Request("POST", "/login",
                    header = [
                        ("Cookie", "TEST_session=DEADBEEF"),
                        ],
                    query = dict(
                        username="fred",
                        password="password1",
                        submit="Sign in",
                        ),
                    ),
                session = dict(
                    session_id = "DEADBEEF",
                    last_openid_request = Stub_OpenIDRequest(
                        http_query = {'openid.mode': "checkid_setup"},
                        params = dict(
                            identity = "http://example.org:0/id/fred",
                            trust_root = "http://example.com/",
                            return_to = "http://example.com/account",
                            ),
                        ),
                    ),
                ),
            }

        logging.basicConfig(stream=self.stdout_test)

        for key, params in self.valid_requests.items():
            args = params.get('args')
            request = params['request']
            opts = Stub_OptionValues(dict(
                host = "example.org",
                port = 0,
                root_url = "http://example.org:0/",
                ))
            gracie_server = Stub_GracieServer(opts)
            sess_manager = gracie_server.sess_manager
            session = params.get('session', dict())
            sess_manager.create_session(session)
            server = params.setdefault(
                'server',
                gracie_server.http_server
                )
            mock_openid_request = self._make_mock_openid_request(
                request.query
                )
            mock_openid_server = self._make_mock_openid_server(
                mock_openid_request
                )
            server.gracie_server.openid_server = mock_openid_server
            if not args:
                args = dict(
                    request = request.connection(),
                    client_address = (opts.host, opts.port),
                    server = server,
                    )
            params['args'] = args

        version = Stub_GracieServer.version
        self.expect_server_version = "Gracie/%(version)s" % vars()
        python_version = sys.version.split()[0]
        self.expect_sys_version = "Python/%(python_version)s" % vars()

    def tearDown(self):
        """ Tear down test fixtures """
        scaffold.mock_restore()

    def _make_mock_openid_request(self, http_query):
        """ Make a mock OpenIDRequest for a given HTTP query """
        openid_request = Mock(
            "OpenIDRequest",
            tracker=self.mock_tracker)
        keys = ('mode', 'identity', 'trust_root', 'return_to')
        query_key_prefix = 'openid.'
        for key in keys:
            query_key = '%(query_key_prefix)s%(key)s' % vars()
            setattr(openid_request, key, http_query.get(query_key))
        openid_request.immediate = (
            openid_request.mode in ['checkid_immediate']
            )
        openid_request.answer.mock_returns = Stub_OpenIDResponse()

        return openid_request

    def _make_mock_openid_server(self, openid_request):
        """ Make a mock OpenIDServer for a given HTTP query """
        def stub_sign(obj):
            return obj

        openid_server = Mock(
            "openid_server",
            tracker=self.mock_tracker)

        if openid_request.mode:
            openid_server.decodeRequest.mock_returns = \
                openid_request
            openid_server.handleRequest.mock_returns = \
                Stub_OpenIDResponse()
            openid_server.encodeResponse.mock_returns = \
                Stub_OpenIDWebResponse()
        openid_server.signatory.sign = stub_sign

        return openid_server

    def test_instantiate(self):
        """ New HTTPRequestHandler instance should be created """
        for params in self.valid_requests.values():
            instance = self.handler_class(**params['args'])
            self.failIfIs(None, instance)

    def test_server_as_specified(self):
        """ HTTPRequestHandler should have specified server attribute """
        for params in self.valid_requests.values():
            instance = self.handler_class(**params['args'])
            server = params['server']
            self.failUnlessEqual(server, instance.server)

    def test_server_version_as_specified(self):
        """ HTTPRequestHandler should report module version """
        for params in self.valid_requests.values():
            instance = self.handler_class(**params['args'])
            self.failUnlessEqual(
                self.expect_server_version, instance.server_version
                )

    def test_version_string_as_specified(self):
        """ HTTPRequestHandler should report expected version string """
        params = self.valid_requests['get-bogus']
        instance = self.handler_class(**params['args'])
        expect_sys_version = self.expect_sys_version
        expect_server_version = self.expect_server_version
        expect_version_string = (
            "%(expect_server_version)s %(expect_sys_version)s"
            ) % vars()
        version_string = instance.version_string()
        self.failUnlessEqual(expect_version_string, version_string)

    def test_command_from_request(self):
        """ Request command attribute should come from request text """
        for params in self.valid_requests.values():
            request = params['request']
            instance = self.handler_class(**params['args'])
            self.failUnlessEqual(request.method, instance.command)

    def test_path_from_request(self):
        """ Request path attribute should come from request text """
        for params in self.valid_requests.values():
            request = params['request']
            instance = self.handler_class(**params['args'])
            self.failUnlessEqual(request.path, instance.path)

    def test_internal_error_sends_server_error_response(self):
        """ When an error condition is raised, should send Server Error """
        class TestingError(StandardError):
            pass
        scaffold.mock(
            "httprequest.HTTPRequestHandler._dispatch",
            raises=TestingError("Testing error"),
            tracker=self.mock_tracker)

        params = self.valid_requests['get-root']
        try:
            instance = self.handler_class(**params['args'])
        except TestingError:
            pass
        expect_mock_output = """\
            ...
            Called httprequest.ResponseHeader(500)
            ...
            Called httprequest.Response.send_to_handler(...)
            """
        self.failUnlessMockCheckerMatch(expect_mock_output)

    def test_request_with_no_cookie_response_creates_session(self):
        """ With no session cookie, response should create new session """
        params = self.valid_requests['no-cookie']
        instance = self.handler_class(**params['args'])
        expect_mock_output = """\
            Called httprequest.ResponseHeader(200)
            ...
            Called httprequest.Response.header.fields.append(
                ('Set-Cookie', 'TEST_session=DEADBEEF'))
            Called httprequest.Response.send_to_handler(...)
            """
        self.failUnlessMockCheckerMatch(expect_mock_output)

    def test_request_with_unknown_cookie_creates_new_session(self):
        """ With unknown username, response should create new session """
        params = self.valid_requests['unknown-cookie']
        instance = self.handler_class(**params['args'])
        expect_mock_output = """\
            Called httprequest.ResponseHeader(200)
            ...
            Called httprequest.Response.header.fields.append(
                ('Set-Cookie', 'TEST_session=DEADBEEF'))
            Called httprequest.Response.send_to_handler(...)
            """
        self.failUnlessMockCheckerMatch(expect_mock_output)

    def test_request_with_good_cookie_response_logged_in(self):
        """ With good session cookie, response should send Logged In """
        params = self.valid_requests['good-cookie']
        identity_name = params['identity_name']
        instance = self.handler_class(**params['args'])
        expect_mock_output = """\
            Called httprequest.ResponseHeader(200)
            ...
            Called httprequest.Response.header.fields.append(
                ('Set-Cookie', 'TEST_session=DEADBEEF-%(identity_name)s'))
            Called httprequest.Response.send_to_handler(...)
            """ % vars()
        self.failUnlessMockCheckerMatch(expect_mock_output)

    def test_get_root_sends_ok_response(self):
        """ Request to GET root document should send OK response """
        params = self.valid_requests['get-root']
        instance = self.handler_class(**params['args'])
        expect_mock_output = """\
            Called httprequest.ResponseHeader(200)
            ...
            Called httprequest.Response.send_to_handler(...)
            """
        self.failUnlessMockCheckerMatch(expect_mock_output)

    def test_get_bogus_url_sends_not_found_response(self):
        """ Request to GET unknown URL should send Not Found response """
        params = self.valid_requests['get-bogus']
        instance = self.handler_class(**params['args'])
        expect_mock_output = """\
            Called httprequest.ResponseHeader(404)
            Called pagetemplate.Page('...')
            ...
            Called httprequest.Response.send_to_handler(...)
            """ % vars()
        self.failUnlessMockCheckerMatch(expect_mock_output)

    def test_get_bogus_identity_sends_not_found_response(self):
        """ Request to GET unknown user should send Not Found response """
        params = self.valid_requests['id-bogus']
        identity_name = params['identity_name']
        instance = self.handler_class(**params['args'])
        expect_mock_output = """\
            Called httprequest.ResponseHeader(404)
            Called pagetemplate.Page('...')
            ...
            Called httprequest.Response.send_to_handler(...)
            """ % vars()
        self.failUnlessMockCheckerMatch(expect_mock_output)

    def test_get_known_identity_sends_view_user_response(self):
        """ Request to GET known user should send view user response """
        params = self.valid_requests['id-fred']
        identity_name = params['identity_name']
        instance = self.handler_class(**params['args'])
        expect_mock_output = """\
            Called httprequest.ResponseHeader(200)
            Called pagetemplate.Page('...')
            ...
            Called httprequest.Response.send_to_handler(...)
            """ % vars()
        self.failUnlessMockCheckerMatch(expect_mock_output)

    def get_logout_creates_new_session_and_redirects(self):
        """ Request to logout should create new session and logout """
        params = self.valid_requests['logout']
        instance = self.handler_class(**params['args'])
        expect_mock_output = """\
            Called httprequest.ResponseHeader(302)
            Called httprequest.ResponseHeader.fields.append('Location', ...)
            Called httprequest.ResponseHeader.fields.append(
                ('Set-Cookie', 'TEST_session=DEADBEEF'))
            Called httprequest.Response.send_to_handler(...)
            """
        self.failUnlessMockCheckerMatch(expect_mock_output)

    def test_get_login_sends_login_form_response(self):
        """ Request to GET login should send login form as response """
        params = self.valid_requests['login']
        instance = self.handler_class(**params['args'])
        expect_mock_output = """\
            Called httprequest.ResponseHeader(200)
            Called pagetemplate.Page('Login')
            ...
            Called httprequest.Response.send_to_handler(...)
            """ % vars()
        self.failUnlessMockCheckerMatch(expect_mock_output)

    def test_post_nobutton_login_sends_not_found_response(self):
        """ POST login with no button should send Not Found response """
        params = self.valid_requests['nobutton-login']
        instance = self.handler_class(**params['args'])
        expect_mock_output = """\
            Called httprequest.ResponseHeader(404)
            ...
            Called httprequest.Response.send_to_handler(...)
            """ % vars()
        self.failUnlessMockCheckerMatch(expect_mock_output)

    def test_post_login_cancel_no_openid_redirects_to_root(self):
        """ Login cancel with no OpenID should redirect to root """
        params = self.valid_requests['cancel-login']
        instance = self.handler_class(**params['args'])
        root_url = params['server'].gracie_server.opts.root_url
        expect_mock_output = """\
            Called httprequest.ResponseHeader(302)
            Called httprequest.ResponseHeader.fields.append(
                ('Location', %(root_url)r))
            ...
            Called httprequest.Response.send_to_handler(...)
            """ % vars()
        self.failUnlessMockCheckerMatch(expect_mock_output)

    def test_post_openid_login_cancel_redirects_to_openid_url(self):
        """ Login cancel with OpenID should redirect to OpenID URL """
        params = self.valid_requests['openid-cancel-login']
        openid_request = params['session']['last_openid_request']
        instance = self.handler_class(**params['args'])
        return_url = openid_request.answer(False).encodeToURL()
        expect_mock_output = """\
            Called httprequest.ResponseHeader(302)
            Called httprequest.ResponseHeader.fields.append(
                ('Location', %(return_url)r))
            ...
            Called httprequest.Response.send_to_handler(...)
            """ % vars()
        self.failUnlessMockCheckerMatch(expect_mock_output)

    def test_post_login_bogus_user_sends_failure_response(self):
        """ POST login with bogus user should send failure response """
        params = self.valid_requests['login-bogus']
        identity_name = params['identity_name']
        instance = self.handler_class(**params['args'])
        expect_mock_output = """\
            Called httprequest.ResponseHeader(200)
            Called pagetemplate.Page('Login Failed')
            ...
            Called httprequest.Response.send_to_handler(...)
            """ % vars()
        self.failUnlessMockCheckerMatch(expect_mock_output)

    def test_post_login_wrong_password_sends_failure_response(self):
        """ POST login with wrong password should send failure response """
        params = self.valid_requests['login-fred-wrong']
        identity_name = params['identity_name']
        instance = self.handler_class(**params['args'])
        expect_mock_output = """\
            Called httprequest.ResponseHeader(200)
            Called pagetemplate.Page('Login Failed')
            ...
            Called httprequest.Response.send_to_handler(...)
            """ % vars()
        self.failUnlessMockCheckerMatch(expect_mock_output)

    def test_post_login_auth_correct_no_openid_redirects_to_root(self):
        """ Login with no OpenID, correct details should redirect to root """
        params = self.valid_requests['login-fred-okay']
        identity_name = params['identity_name']
        root_url = params['server'].gracie_server.opts.root_url
        instance = self.handler_class(**params['args'])
        expect_mock_output = """\
            Called httprequest.ResponseHeader(302)
            Called httprequest.ResponseHeader.fields.append(
                ('Location', %(root_url)r))
            ...
            Called httprequest.Response.send_to_handler(
                ...)
            """ % vars()
        self.failUnlessMockCheckerMatch(expect_mock_output)

    def test_post_openid_login_auth_correct_redirects_to_openid_url(self):
        """ Login correct with OpenID should redirect to OpenID URL """
        params = self.valid_requests['openid-login-fred-okay']
        openid_request = params['session']['last_openid_request']
        instance = self.handler_class(**params['args'])
        return_url = openid_request.answer(True).encodeToURL()
        expect_mock_output = """\
            Called httprequest.ResponseHeader(302)
            Called httprequest.ResponseHeader.fields.append(
                ('Location', %(return_url)r))
            ...
            Called httprequest.Response.send_to_handler(...)
            """ % vars()
        self.failUnlessMockCheckerMatch(expect_mock_output)

    def test_post_openid_login_auth_other_sends_wrong_auth(self):
        """ Login wrong auth with OpenID should send Auth Required """
        params = self.valid_requests['openid-login-bill-other']
        instance = self.handler_class(**params['args'])
        expect_mock_output = """\
            Called httprequest.ResponseHeader(200)
            Called pagetemplate.Page('Authentication Required')
            ...
            Called httprequest.Response.send_to_handler(...)
            """ % vars()
        self.failUnlessMockCheckerMatch(expect_mock_output)

    def test_get_server_no_query_sends_about_site_response(self):
        """ GET to server without query should send About page """
        params = self.valid_requests['openid-no-query']
        instance = self.handler_class(**params['args'])
        expect_mock_output = """\
            Called openid_server.decodeRequest(...)
            Called httprequest.ResponseHeader(200)
            Called pagetemplate.Page('About this site')
            ...
            Called httprequest.Response.send_to_handler(...)
            """
        self.failUnlessMockCheckerMatch(expect_mock_output)

    def test_get_server_bogus_query_sends_server_error_response(self):
        """ GET to server with bogus query should send Server Error """
        def raise_ProtocolError(_):
            raise Stub_OpenIDError("Testing error")

        params = self.valid_requests['openid-bogus-query']
        args = params['args']
        server = args['server']
        server.gracie_server.openid_server.decodeRequest = \
            raise_ProtocolError
        try:
            instance = self.handler_class(**args)
        except Stub_OpenIDError:
            pass
        expect_mock_output = """\
            Called httprequest.ResponseHeader(500)
            ...
            Called httprequest.Response.send_to_handler(...)
            """
        self.failUnlessMockCheckerMatch(expect_mock_output)

    def test_get_server_assoc_query_delegates_to_openid(self):
        """ OpenID associate query should be passed to openid server """
        params = self.valid_requests['openid-query-associate']
        args = params['args']
        server = args['server']
        instance = self.handler_class(**args)
        expect_mock_output = """\
            Called openid_server.decodeRequest(...)
            Called openid_server.handleRequest(...)
            Called openid_server.encodeResponse(...)
            Called httprequest.ResponseHeader(200)
            Called httprequest.ResponseHeader.fields.append(('openid', 'yes'))
            Called httprequest.Response(..., 'OpenID response')
            ...
            Called httprequest.Response.send_to_handler(...)
            """
        self.failUnlessMockCheckerMatch(expect_mock_output)

    def test_checkid_immediate_no_session_returns_failure(self):
        """ OpenID check_immediate with no session should reject """
        params_key = 'openid-query-checkid_immediate-no-session'
        params = self.valid_requests[params_key]
        args = params['args']
        server = args['server']
        instance = self.handler_class(**args)
        expect_mock_output = """\
            Called openid_server.decodeRequest(...)
            Called OpenIDRequest.answer(False, ...)
            Called openid_server.encodeResponse(...)
            ...
            Called httprequest.Response.send_to_handler(...)
            """
        self.failUnlessMockCheckerMatch(expect_mock_output)

    def test_checkid_setup_wrong_session_returns_wrong_auth(self):
        """ OpenID checkid_setup with wrong session should request login """
        for params_key in [
            'openid-query-checkid_setup-no-session',
            'openid-query-checkid_setup-other-session',
            ]:
            params = self.valid_requests[params_key]
            args = params['args']
            server = args['server']
            instance = self.handler_class(**args)
            expect_mock_output = """\
                Called openid_server.decodeRequest(...)
                Called httprequest.ResponseHeader(200)
                Called pagetemplate.Page('Authentication Required')
                ...
                Called httprequest.Response.send_to_handler(...)
                """
            self.failUnlessMockCheckerMatch(expect_mock_output)
            self.mock_tracker.clear()

    def test_checkid_with_session_returns_success(self):
        """ OpenID checkid with right session should succeed """
        for params_key in [
            'openid-query-checkid_immediate-right-session',
            'openid-query-checkid_setup-right-session',
            ]:
            params = self.valid_requests[params_key]
            args = params['args']
            server = args['server']
            instance = self.handler_class(**args)
            expect_mock_output = """\
                Called openid_server.decodeRequest(...)
                Called OpenIDRequest.answer(True)
                Called openid_server.encodeResponse(...)
                ...
                Called httprequest.Response.send_to_handler(...)
                """
            self.failUnlessMockCheckerMatch(expect_mock_output)
            self.mock_tracker.clear()
